@@ -1,21 +1,25 @@
 # Test methods with long descriptive names can omit docstrings
 # pylint: disable=missing-docstring
-
+# pylint: disable=protected-access
+import os
 import sys
 import math
 import unittest
 import pickle
 import pkgutil
+import warnings
 from datetime import datetime, timezone
 
 from io import StringIO
 
 import numpy as np
+import pandas as pd
 import scipy.sparse as sp
 
 from Orange.data import Variable, ContinuousVariable, DiscreteVariable, \
-    StringVariable, TimeVariable, Unknown, Value
+    StringVariable, TimeVariable, Unknown, Value, Table
 from Orange.data.io import CSVReader
+from Orange.preprocess.transformation import Identity
 from Orange.tests.base import create_pickling_tests
 from Orange.util import OrangeDeprecationWarning
 
@@ -33,11 +37,10 @@ def is_on_path(name):
     -------
     found : bool
     """
-    for loader, name_, ispkg in pkgutil.iter_modules(sys.path):
+    for _, name_, _ in pkgutil.iter_modules(sys.path):
         if name == name_:
             return True
-    else:
-        return False
+    return False
 
 
 # noinspection PyPep8Naming,PyUnresolvedReferences
@@ -142,6 +145,96 @@ class TestVariable(unittest.TestCase):
         self.assertNotEqual(a, "somestring")
         self.assertEqual(hash(a), hash(b))
 
+    def test_eq_with_compute_value(self):
+        a = ContinuousVariable("a")
+        b = ContinuousVariable("a")
+        self.assertEqual(a, a)
+        self.assertEqual(a, b)
+        self.assertIsNot(a, b)
+
+        a._compute_value = lambda x: x
+        self.assertEqual(a, a)
+        self.assertNotEqual(a, b)
+
+        a1 = ContinuousVariable("a")
+        a2 = ContinuousVariable("a")
+        c = ContinuousVariable("c")
+
+        a._compute_value = Identity(a1)
+        self.assertEqual(a, a)
+        self.assertEqual(a, b)
+        self.assertEqual(hash(a), hash(b))
+
+        b._compute_value = a.compute_value
+        self.assertEqual(a, b)
+
+        b._compute_value = Identity(a1)
+        self.assertEqual(a, b)
+
+        b._compute_value = Identity(a2)
+        self.assertEqual(a, b)
+
+        b._compute_value = Identity(c)
+        self.assertNotEqual(a, b)
+
+        b._compute_value = Identity(a2)
+        a1._compute_value = lambda x: x
+        self.assertNotEqual(a, b)
+
+        a1._compute_value = Identity(c)
+        self.assertNotEqual(a, b)
+
+        a2._compute_value = Identity(c)
+        self.assertEqual(a, b)
+
+    def test_hash(self):
+        a = ContinuousVariable("a")
+        b = ContinuousVariable("a")
+        self.assertEqual(hash(a), hash(b))
+
+        a._compute_value = lambda x: x
+        self.assertNotEqual(hash(a), hash(b))
+
+        b._compute_value = lambda x: x
+        self.assertNotEqual(hash(a), hash(b))
+
+        a1 = ContinuousVariable("a")
+        a2 = ContinuousVariable("a")
+
+        a._compute_value = Identity(a1)
+        self.assertNotEqual(hash(a), hash(b))
+
+        b._compute_value = Identity(a2)
+        self.assertEqual(hash(a), hash(b))
+
+        at = TimeVariable("a")
+        b = ContinuousVariable("b")
+        self.assertEqual(hash(a1), hash(a2))
+        self.assertNotEqual(hash(a1), hash(b))
+        self.assertNotEqual(hash(a1), hash(at))
+
+    def test_hash_eq(self):
+        a = ContinuousVariable("a")
+        a1 = ContinuousVariable("a")
+        b1 = ContinuousVariable("b", compute_value=Identity(a))
+        b2 = ContinuousVariable("b2", compute_value=Identity(b1))
+        b3 = ContinuousVariable("b")
+        c1 = ContinuousVariable("c", compute_value=Identity(a))
+        c2 = ContinuousVariable("c", compute_value=Identity(a))
+        self.assertNotEqual(a, b2)
+        self.assertNotEqual(b1, b2)
+        self.assertNotEqual(a, b1)
+        self.assertNotEqual(b1, b3)
+        self.assertEqual(a, a1)
+        self.assertEqual(c1, c2)
+
+        self.assertNotEqual(hash(a), hash(b2))
+        self.assertNotEqual(hash(b1), hash(b2))
+        self.assertNotEqual(hash(a), hash(b1))
+        self.assertNotEqual(hash(b1), hash(b3))
+        self.assertEqual(hash(a), hash(a1))
+        self.assertEqual(hash(c1), hash(c2))
+
 
 def variabletest(varcls):
     def decorate(cls):
@@ -165,40 +258,56 @@ class TestDiscreteVariable(VariableTest):
             var.to_val("G")
 
     def test_make(self):
-        var = DiscreteVariable.make("a", values=["F", "M"])
+        var = DiscreteVariable.make("a", values=("F", "M"))
         self.assertIsInstance(var, DiscreteVariable)
         self.assertEqual(var.name, "a")
-        self.assertEqual(var.values, ["F", "M"])
+        self.assertEqual(var.values, ("F", "M"))
 
     def test_val_from_str(self):
-        var = DiscreteVariable.make("a", values=["F", "M"])
+        var = DiscreteVariable.make("a", values=("F", "M"))
         self.assertTrue(math.isnan(var.to_val(None)))
         self.assertEqual(var.to_val(1), 1)
 
+    def test_val_from_str_add(self):
+        var = DiscreteVariable.make("a", values=("F", "M"))
+        self.assertTrue(math.isnan(var.val_from_str_add(None)))
+        self.assertEqual(var.val_from_str_add("M"), 1)
+        self.assertEqual(var.val_from_str_add("F"), 0)
+        self.assertEqual(var.values, ("F", "M"))
+        self.assertEqual(var.val_from_str_add("N"), 2)
+        self.assertEqual(var.values, ("F", "M", "N"))
+        self.assertEqual(var._value_index, {"F": 0, "M": 1, "N": 2})
+        self.assertEqual(var.val_from_str_add("M"), 1)
+        self.assertEqual(var.val_from_str_add("F"), 0)
+        self.assertEqual(var.val_from_str_add("N"), 2)
+
     def test_repr(self):
-        var = DiscreteVariable.make("a", values=["F", "M"])
+        var = DiscreteVariable.make("a", values=("F", "M"))
         self.assertEqual(
             repr(var),
-            "DiscreteVariable(name='a', values=['F', 'M'])")
-        var.ordered = True
-        self.assertEqual(
-            repr(var),
-            "DiscreteVariable(name='a', values=['F', 'M'], ordered=True)")
+            "DiscreteVariable(name='a', values=('F', 'M'))")
 
         var = DiscreteVariable.make("a", values="1234567")
         self.assertEqual(
             repr(var),
-            "DiscreteVariable(name='a', values=['1', '2', '3', '4', '5', '6', '7'])")
+            "DiscreteVariable(name='a', values=('1', '2', '3', '4', '5', '6', '7'))"
+        )
 
     def test_no_nonstringvalues(self):
-        self.assertRaises(TypeError, DiscreteVariable, "foo", values=["a", 42])
-        a = DiscreteVariable("foo", values=["a", "b", "c"])
+        self.assertRaises(TypeError, DiscreteVariable, "foo", values=("a", 42))
+        a = DiscreteVariable("foo", values=("a", "b", "c"))
         self.assertRaises(TypeError, a.add_value, 42)
 
+    def test_no_duplicated_values(self):
+        a = DiscreteVariable("foo", values=["a", "b", "c"])
+        a.add_value("b")
+        self.assertEqual(list(a.values), ["a", "b", "c"])
+        self.assertEqual(list(a._value_index), ["a", "b", "c"])
+
     def test_unpickle(self):
-        d1 = DiscreteVariable("A", values=["two", "one"])
+        d1 = DiscreteVariable("A", values=("two", "one"))
         s = pickle.dumps(d1)
-        d2 = DiscreteVariable.make("A", values=["one", "two", "three"])
+        d2 = DiscreteVariable.make("A", values=("one", "two", "three"))
         d2_values = tuple(d2.values)
         d1c = pickle.loads(s)
         # See: gh-3238
@@ -207,14 +316,14 @@ class TestDiscreteVariable(VariableTest):
         self.assertSequenceEqual(d2.values, d2_values)
         self.assertSequenceEqual(d1c.values, d1.values)
         s = pickle.dumps(d2)
-        d1 = DiscreteVariable("A", values=["one", "two"])
+        d1 = DiscreteVariable("A", values=("one", "two"))
         d2 = pickle.loads(s)
-        self.assertSequenceEqual(d2.values, ["one", "two", "three"])
-        self.assertSequenceEqual(d1.values, ["one", "two"])
+        self.assertSequenceEqual(d2.values, ("one", "two", "three"))
+        self.assertSequenceEqual(d1.values, ("one", "two"))
 
     def test_mapper_dense(self):
-        abc = DiscreteVariable("a", values=list("abc"))
-        dca = DiscreteVariable("a", values=list("dca"))
+        abc = DiscreteVariable("a", values=tuple("abc"))
+        dca = DiscreteVariable("a", values=tuple("dca"))
         mapper = dca.get_mapper_from(abc)
 
         self.assertEqual(mapper(0), 2)
@@ -248,18 +357,21 @@ class TestDiscreteVariable(VariableTest):
 
         arr_list = list(arr)
         self.assertIsNot(mapper(arr_list), arr_list)
-        self.assertTrue(all(x == y or (x != x and y != y) for x, y in zip(
-            mapper(arr_list), [2, 2, 1, np.nan, 2, np.nan, np.nan])))
+        self.assertTrue(
+            all(x == y or (np.isnan(x) and np.isnan(y))
+                for x, y in zip(mapper(arr_list),
+                                [2, 2, 1, np.nan, 2, np.nan, np.nan])))
 
-        self.assertTrue(x == y or (x != x and y != y) for x, y in zip(
-            mapper(tuple(arr)),
-            (2, 2, 1, np.nan, 2, np.nan, np.nan)))
+        self.assertTrue(
+            x == y or (np.isnan(x) and np.isnan(y))
+            for x, y in zip(mapper(tuple(arr)),
+                            (2, 2, 1, np.nan, 2, np.nan, np.nan)))
 
         self.assertRaises(ValueError, mapper, object())
 
     def test_mapper_sparse(self):
-        abc = DiscreteVariable("a", values=list("abc"))
-        dca = DiscreteVariable("a", values=list("dca"))
+        abc = DiscreteVariable("a", values=tuple("abc"))
+        dca = DiscreteVariable("a", values=tuple("dca"))
         mapper = dca.get_mapper_from(abc)
 
         arr = np.array([0, 0, 2, 1, 0, 1, np.nan])
@@ -278,7 +390,7 @@ class TestDiscreteVariable(VariableTest):
             np.array([2, 2, 1, np.nan, 2, np.nan, np.nan]))
 
         # 0 maps to 0 -> keep sparse
-        acd = DiscreteVariable("a", values=list("acd"))
+        acd = DiscreteVariable("a", values=tuple("acd"))
         mapper = acd.get_mapper_from(abc)
 
         arr_csr = sp.csr_matrix(arr)
@@ -299,8 +411,8 @@ class TestDiscreteVariable(VariableTest):
 
     def test_mapper_inplace(self):
         s = list(range(7))
-        abc = DiscreteVariable("a", values=list("abc"))
-        dca = DiscreteVariable("a", values=list("dca"))
+        abc = DiscreteVariable("a", values=tuple("abc"))
+        dca = DiscreteVariable("a", values=tuple("dca"))
         mapper = dca.get_mapper_from(abc)
 
         arr = np.array([[0, 0, 2, 1, 0, 1, np.nan], s]).T
@@ -312,7 +424,7 @@ class TestDiscreteVariable(VariableTest):
         self.assertRaises(ValueError, mapper, [1, 2, 3], 0)
         self.assertRaises(ValueError, mapper, 1, 0)
 
-        acd = DiscreteVariable("a", values=list("acd"))
+        acd = DiscreteVariable("a", values=tuple("acd"))
         mapper = acd.get_mapper_from(abc)
 
         arr = np.array([[0, 0, 2, 1, 0, 1, np.nan], s]).T
@@ -333,8 +445,8 @@ class TestDiscreteVariable(VariableTest):
             np.array([[0, 0, 1, np.nan, 0, np.nan, np.nan], s]).T)
 
     def test_mapper_dim_check(self):
-        abc = DiscreteVariable("a", values=list("abc"))
-        dca = DiscreteVariable("a", values=list("dca"))
+        abc = DiscreteVariable("a", values=tuple("abc"))
+        dca = DiscreteVariable("a", values=tuple("dca"))
         mapper = dca.get_mapper_from(abc)
 
         self.assertRaises(ValueError, mapper, np.zeros((7, 2)))
@@ -342,8 +454,8 @@ class TestDiscreteVariable(VariableTest):
         self.assertRaises(ValueError, mapper, sp.csc_matrix(np.zeros((7, 2))))
 
     def test_mapper_from_no_values(self):
-        abc = DiscreteVariable("a", values=[])
-        dca = DiscreteVariable("a", values=list("dca"))
+        abc = DiscreteVariable("a", values=())
+        dca = DiscreteVariable("a", values=tuple("dca"))
         mapper = dca.get_mapper_from(abc)
 
         arr = np.full(7, np.nan)
@@ -365,26 +477,44 @@ class TestDiscreteVariable(VariableTest):
 
     def varcls_modified(self, name):
         var = super().varcls_modified(name)
-        var.values = ["A", "B"]
-        var.ordered = True
+        var.add_value("A")
+        var.add_value("B")
         return var
 
     def test_copy_checks_len_values(self):
-        var = DiscreteVariable("gender", values=["F", "M"])
-        self.assertEqual(var.values, ["F", "M"])
+        var = DiscreteVariable("gender", values=("F", "M"))
+        self.assertEqual(var.values, ("F", "M"))
 
-        self.assertRaises(ValueError, var.copy, values=["F", "M", "N"])
-        self.assertRaises(ValueError, var.copy, values=["F"])
-        self.assertRaises(ValueError, var.copy, values=[])
+        self.assertRaises(ValueError, var.copy, values=("F", "M", "N"))
+        self.assertRaises(ValueError, var.copy, values=("F",))
+        self.assertRaises(ValueError, var.copy, values=())
 
         var2 = var.copy()
-        self.assertEqual(var2.values, ["F", "M"])
+        self.assertEqual(var2.values, ("F", "M"))
 
         var2 = var.copy(values=None)
-        self.assertEqual(var2.values, ["F", "M"])
+        self.assertEqual(var2.values, ("F", "M"))
 
-        var2 = var.copy(values=["W", "M"])
-        self.assertEqual(var2.values, ["W", "M"])
+        var2 = var.copy(values=("W", "M"))
+        self.assertEqual(var2.values, ("W", "M"))
+
+    def test_pickle_backward_compatibility(self):
+        """
+        Test that pickle made with an older version of Orange are correctly
+        loaded after changes in DiscreteVariable
+        """
+        with warnings.catch_warnings():
+            # travis/gh-action tests change OrangeDeprecationWarning to error
+            # temporary disable it
+            warnings.simplefilter('default', OrangeDeprecationWarning)
+            this_dir = os.path.dirname(os.path.realpath(__file__))
+            datasets_dir = os.path.join(
+                this_dir, "..", "..", "tests", "datasets"
+            )
+            # pickle with values as list
+            Table(os.path.join(datasets_dir, "sailing-orange-3-20.pkl"))
+            # pickle with values as tuple list
+            Table(os.path.join(datasets_dir, "iris-orange-3-25.pkl"))
 
 
 @variabletest(ContinuousVariable)
@@ -397,23 +527,42 @@ class TestContinuousVariable(VariableTest):
 
     def test_decimals(self):
         a = ContinuousVariable("a", 4)
-        self.assertEqual(a.str_val(4.654321), "4.6543")
-        self.assertEqual(a.str_val(4.654321654321), "4.6543")
+        self.assertEqual(a.str_val(4.6543), "4.6543")
+        self.assertEqual(a.str_val(4.25), "4.2500")
         self.assertEqual(a.str_val(Unknown), "?")
         a = ContinuousVariable("a", 5)
         self.assertEqual(a.str_val(0.000000000001), "0.00000")
         a = ContinuousVariable("a", 10)
         self.assertEqual(a.str_val(0.000000000001), "1e-12")
 
+    def test_more_decimals(self):
+        a = ContinuousVariable("a", 0)
+        self.assertEqual(a.str_val(4), "4")
+        self.assertEqual(a.str_val(4.1234), "4.12")
+
+        a = ContinuousVariable("a", 2)
+        self.assertEqual(a.str_val(4), "4.00")
+        self.assertEqual(a.str_val(4.25), "4.25")
+        self.assertEqual(a.str_val(4.1234123), "4.1234")
+
+        for cca4 in (4 + 1e-9, 4 - 1e-9):
+            assert cca4 != 4
+            self.assertEqual(a.str_val(cca4), "4.00")
+
     def test_adjust_decimals(self):
+        # Default is 3 decimals, but format is %g
         a = ContinuousVariable("a")
+        self.assertEqual(a.str_val(5), "5")
         self.assertEqual(a.str_val(4.65432), "4.65432")
+
+        # Change to no decimals
         a.val_from_str_add("5")
-        self.assertEqual(a.str_val(4.65432), "5")
+        self.assertEqual(a.str_val(5), "5")
+
+        # Change to two decimals
         a.val_from_str_add("  5.12    ")
-        self.assertEqual(a.str_val(4.65432), "4.65")
-        a.val_from_str_add("5.1234")
-        self.assertEqual(a.str_val(4.65432), "4.6543")
+        self.assertEqual(a.str_val(4.65), "4.65")
+        self.assertEqual(a.str_val(5), "5.00")
 
     def varcls_modified(self, name):
         var = super().varcls_modified(name)
@@ -482,6 +631,10 @@ class TestTimeVariable(VariableTest):
         ts2 = var.parse(datestr)
         self.assertEqual(var.repr_val(ts2), datestr)
         self.assertEqual(var.repr_val(ts1), '2015-10-18 20:48:20')
+        # TZ is reset to UTC.
+        datestr, offset = '2015-10-18T22:48:20', '+02:00'
+        ts3 = var.parse(datestr + offset)
+        self.assertEqual(var.repr_val(ts3), '2015-10-18 20:48:20')
 
     def test_parse_timestamp(self):
         var = TimeVariable("time")
@@ -546,6 +699,117 @@ time,continuous
         var.have_time = 1
         return var
 
+    def test_additional_formats(self):
+        expected_date = datetime(2022, 2, 7)
+        dates = {
+            "2021-11-25": ("2022-02-07",),
+            "25.11.2021": ("07.02.2022", "07. 02. 2022", "7.2.2022", "7. 2. 2022"),
+            "25.11.21": ("07.02.22", "07. 02. 22", "7.2.22", "7. 2. 22"),
+            "11/25/2021": ("02/07/2022", "2/7/2022"),
+            "11/25/21": ("02/07/22", "2/7/22"),
+            "20211125": ("20220207",),
+        }
+        expected_date_time = datetime(2022, 2, 7, 10, 11, 12)
+        date_times = {
+            "2021-11-25 00:00:00": (
+                "2022-02-07 10:11:12",
+                "2022-02-07 10:11:12.00",
+            ),
+            "25.11.2021 00:00:00": (
+                "07.02.2022 10:11:12",
+                "07. 02. 2022 10:11:12",
+                "7.2.2022 10:11:12",
+                "7. 2. 2022 10:11:12",
+                "07.02.2022 10:11:12.00",
+                "07. 02. 2022 10:11:12.00",
+                "7.2.2022 10:11:12.00",
+                "7. 2. 2022 10:11:12.00",
+            ),
+            "25.11.21 00:00:00": (
+                "07.02.22 10:11:12",
+                "07. 02. 22 10:11:12",
+                "7.2.22 10:11:12",
+                "7. 2. 22 10:11:12",
+                "07.02.22 10:11:12.00",
+                "07. 02. 22 10:11:12.00",
+                "7.2.22 10:11:12.00",
+                "7. 2. 22 10:11:12.00",
+            ),
+            "11/25/2021 00:00:00": (
+                "02/07/2022 10:11:12",
+                "2/7/2022 10:11:12",
+                "02/07/2022 10:11:12.00",
+                "2/7/2022 10:11:12.00",
+            ),
+            "11/25/21 00:00:00": (
+                "02/07/22 10:11:12",
+                "2/7/22 10:11:12",
+                "02/07/22 10:11:12.00",
+                "2/7/22 10:11:12.00",
+            ),
+            "20211125000000": ("20220207101112", "20220207101112.00"),
+        }
+        # times without seconds
+        expected_date_time2 = datetime(2022, 2, 7, 10, 11, 0)
+        date_times2 = {
+            "2021-11-25 00:00:00": ("2022-02-07 10:11",),
+            "25.11.2021 00:00:00": (
+                "07.02.2022 10:11",
+                "07. 02. 2022 10:11",
+                "7.2.2022 10:11",
+                "7. 2. 2022 10:11",
+            ),
+            "25.11.21 00:00:00": (
+                "07.02.22 10:11",
+                "07. 02. 22 10:11",
+                "7.2.22 10:11",
+                "7. 2. 22 10:11",
+            ),
+            "11/25/2021 00:00:00": ("02/07/2022 10:11", "2/7/2022 10:11"),
+            "11/25/21 00:00:00": ("02/07/22 10:11", "2/7/22 10:11"),
+            "20211125000000": ("202202071011",),
+        }
+        # datetime defaults to 1900, 01, 01
+        expected_time = datetime(1900, 1, 1, 10, 11, 12)
+        times = {
+            "00:00:00": ("10:11:12", "10:11:12.00"),
+            "000000": ("101112", "101112.00"),
+        }
+        expected_time2 = datetime(1900, 1, 1, 10, 11, 0)
+        times2 = {
+            "00:00:00": ("10:11",),
+        }
+        expected_year = datetime(2022, 1, 1)
+        years = {
+            "2021": (2022,),
+        }
+        expected_day = datetime(1900, 2, 7)
+        days = {
+            "11-25": ("02-07",),
+            "25.11.": ("07.02.", "07. 02.", "7.2.", "7. 2."),
+            "11/25": ("02/07", "2/7"),
+        }
+        data = (
+            (expected_date, dates),
+            (expected_date_time, date_times),
+            (expected_date_time2, date_times2),
+            (expected_time, times),
+            (expected_time2, times2),
+            (expected_year, years),
+            (expected_day, days),
+        )
+        for expected, dts in data:
+            for k, dt in dts.items():
+                for t in dt:
+                    parsed = [
+                        pd.to_datetime(t, format=f, errors="coerce")
+                        for f in TimeVariable.ADDITIONAL_FORMATS[k][0]
+                    ]
+                    # test any equal to expected
+                    self.assertTrue(any(d == expected for d in parsed))
+                    # test that no other equal to any other date - only nan or expected
+                    self.assertTrue(any(d == expected or pd.isnull(d) for d in parsed))
+
 
 PickleContinuousVariable = create_pickling_tests(
     "PickleContinuousVariable",
@@ -556,10 +820,7 @@ PickleDiscreteVariable = create_pickling_tests(
     "PickleDiscreteVariable",
     ("with_name", lambda: DiscreteVariable(name="Feature 0")),
     ("with_str_value", lambda: DiscreteVariable(name="Feature 0",
-                                                values=["F", "M"])),
-    ("ordered", lambda: DiscreteVariable(name="Feature 0",
-                                         values=["F", "M"],
-                                         ordered=True)),
+                                                values=("F", "M")))
 )
 
 
@@ -571,7 +832,7 @@ PickleStringVariable = create_pickling_tests(
 
 class VariableTestMakeProxy(unittest.TestCase):
     def test_make_proxy_disc(self):
-        abc = DiscreteVariable("abc", values="abc", ordered=True)
+        abc = DiscreteVariable("abc", values="abc")
         abc1 = abc.make_proxy()
         abc2 = abc1.make_proxy()
         self.assertEqual(abc, abc1)
@@ -580,7 +841,7 @@ class VariableTestMakeProxy(unittest.TestCase):
         self.assertEqual(hash(abc), hash(abc1))
         self.assertEqual(hash(abc1), hash(abc2))
 
-        abcx = DiscreteVariable("abc", values="abc", ordered=True)
+        abcx = DiscreteVariable("abc", values="abc")
         self.assertEqual(abc, abcx)
         self.assertIsNot(abc, abcx)
 
