@@ -1,6 +1,6 @@
 import re
 import warnings
-from collections import Iterable
+from collections.abc import Iterable
 
 from datetime import datetime, timedelta, timezone
 from numbers import Number, Real, Integral
@@ -34,7 +34,10 @@ def make_variable(cls, compute_value, *args):
     if compute_value is not None:
         return cls(*args, compute_value=compute_value)
     else:
-        # For compatibility with old pickles
+        # For compatibility with old pickles: remove the second arg if it's
+        # bool `compute_value` (args[3]) can't be bool, so this should be safe
+        if len(args) > 2 and isinstance(args[2], bool):
+            args = args[:2] + args[3:]
         return cls(*args)
 
 
@@ -47,13 +50,14 @@ def is_discrete_values(values):
     ----
     Assumes consistent type of items of `values`.
     """
-    if not len(values):
+    if len(values) == 0:
         return None
     # If the first few values are, or can be converted to, floats,
     # the type is numeric
     try:
         isinstance(next(iter(values)), Number) or \
-        [float(v) for _, v in zip(range(min(3, len(values))), values)]
+        [v not in MISSING_VALUES and float(v)
+         for _, v in zip(range(min(3, len(values))), values)]
     except ValueError:
         is_numeric = False
         max_values = int(round(len(values)**.7))
@@ -165,6 +169,7 @@ class Value(float):
         return self
 
     def __init__(self, _, __=Unknown):
+        # __new__ does the job, pylint: disable=super-init-not-called
         pass
 
     def __repr__(self):
@@ -176,8 +181,10 @@ class Value(float):
 
     def __eq__(self, other):
         if isinstance(self, Real) and isnan(self):
-            return (isinstance(other, Real) and isnan(other)
-                    or other in self.variable.unknown_str)
+            if isinstance(other, Real):
+                return isnan(other)
+            else:
+                return other in self.variable.unknown_str
         if isinstance(other, str):
             return self.variable.str_val(self) == other
         if isinstance(other, Value):
@@ -219,7 +226,9 @@ class Value(float):
         if self.variable.is_discrete:
             # It is not possible to hash the id and the domain value to the
             # same number as required by __eq__.
-            # hash(1) == hash(Value(DiscreteVariable("var", ["red", "green", "blue"]), 1)) == hash("green")
+            # hash(1)
+            # == hash(Value(DiscreteVariable("var", ["red", "green", "blue"]), 1))
+            # == hash("green")
             # User should hash directly ids or domain values instead.
             raise TypeError("unhashable type - cannot hash values of discrete variables!")
         if self._value is None:
@@ -242,6 +251,7 @@ class Value(float):
         return dict(value=getattr(self, '_value', None))
 
     def __setstate__(self, state):
+        # defined in __new__, pylint: disable=attribute-defined-outside-init
         self._value = state.get('value', None)
 
 
@@ -341,12 +351,29 @@ class Variable(Reprable, metaclass=VariableMeta):
         return var
 
     def __eq__(self, other):
-        return type(self) is type(other) \
-               and self.name == other.name \
-               and self._compute_value == other._compute_value
+        if type(self) is not type(other):
+            return False
+
+        var1 = self._get_identical_source(self)
+        var2 = self._get_identical_source(other)
+        # pylint: disable=protected-access
+        return (
+            self.name == other.name
+            and var1.name == var2.name
+            and var1._compute_value == var2._compute_value
+        )
 
     def __hash__(self):
-        return hash((self.name, type(self), self._compute_value))
+        var = self._get_identical_source(self)
+        return hash((self.name, var.name, type(self), var._compute_value))
+
+    @staticmethod
+    def _get_identical_source(var):
+        # pylint: disable=protected-access,import-outside-toplevel
+        from Orange.preprocess.transformation import Identity
+        while isinstance(var._compute_value, Identity):
+            var = var._compute_value.variable
+        return var
 
     @classmethod
     def make(cls, name, *args, **kwargs):
@@ -391,7 +418,8 @@ class Variable(Reprable, metaclass=VariableMeta):
     def is_time(self):
         return isinstance(self, TimeVariable)
 
-    def repr_val(self, val):
+    @staticmethod
+    def repr_val(val):
         """
         Return a textual representation of variable's value `val`. Argument
         `val` must be a float (for primitive variables) or an arbitrary
@@ -453,13 +481,21 @@ class Variable(Reprable, metaclass=VariableMeta):
         # Use make to unpickle variables.
         return make_variable, (self.__class__, self._compute_value, self.name), self.__dict__
 
-    def copy(self, compute_value=None, *, name=None, **kwargs):
+    _CopyComputeValue = object()
+
+    def copy(self, compute_value=_CopyComputeValue, *, name=None, **kwargs):
+        if compute_value is self._CopyComputeValue:
+            compute_value = self.compute_value
         var = type(self)(name=name or self.name,
-                         compute_value=compute_value or self.compute_value,
+                         compute_value=compute_value,
                          sparse=self.sparse, **kwargs)
         var.attributes = dict(self.attributes)
         return var
 
+    def renamed(self, new_name):
+        # prevent cyclic import, pylint: disable=import-outside-toplevel
+        from Orange.preprocess.transformation import Identity
+        return self.copy(name=new_name, compute_value=Identity(variable=self))
 
 del _predicatedescriptor
 
@@ -496,6 +532,7 @@ class ContinuousVariable(Variable):
         three, but adjusted at the first call of :obj:`to_val`.
         """
         super().__init__(name, compute_value, sparse=sparse)
+        self._max_round_diff = 0
         self.number_of_decimals = number_of_decimals
 
     @property
@@ -520,6 +557,7 @@ class ContinuousVariable(Variable):
             return
 
         self._number_of_decimals = x
+        self._max_round_diff = 10 ** (-x - 6)
         self.adjust_decimals = 0
         if self._number_of_decimals <= MAX_NUM_OF_DECIMALS:
             self._format_str = "%.{}f".format(self.number_of_decimals)
@@ -545,21 +583,36 @@ class ContinuousVariable(Variable):
         """
         Return the value as a string with the prescribed number of decimals.
         """
-        if isnan(val):
+        # Table value can't be inf, but repr_val can be used to print any float
+        if not np.isfinite(val):
             return "?"
+        if self.format_str != "%g" \
+                and abs(round(val, self._number_of_decimals) - val) \
+                > self._max_round_diff:
+            return f"{val:.{self._number_of_decimals + 2}f}"
         return self._format_str % val
 
     str_val = repr_val
 
-    def copy(self, compute_value=None, *, name=None, **kwargs):
-        var = super().copy(compute_value=compute_value, name=name,
-                           number_of_decimals=self.number_of_decimals,
-                           **kwargs)
-        var.adjust_decimals = self.adjust_decimals
-        var.format_str = self._format_str
+    def copy(self, compute_value=Variable._CopyComputeValue,
+             *, name=None, **kwargs):
+        # pylint understand not that `var` is `DiscreteVariable`:
+        # pylint: disable=protected-access
+        number_of_decimals = kwargs.pop("number_of_decimals", None)
+        var = super().copy(compute_value=compute_value, name=name, **kwargs)
+        if number_of_decimals is not None:
+            var.number_of_decimals = number_of_decimals
+        else:
+            var._number_of_decimals = self._number_of_decimals
+            var._max_round_diff = self._max_round_diff
+            var.adjust_decimals = self.adjust_decimals
+            var.format_str = self._format_str
         return var
 
 
+TupleList = tuple # backward compatibility (for pickled table)
+
+
 class DiscreteVariable(Variable):
     """
     Descriptor for symbolic, discrete variables. Values of discrete variables
@@ -569,32 +622,32 @@ class DiscreteVariable(Variable):
     .. attribute:: values
 
         A list of variable's values.
-
-    .. attribute:: ordered
-
-        Some algorithms (and, in particular, visualizations) may
-        sometime reorder the values of the variable, e.g. alphabetically.
-        This flag hints that the given order of values is "natural"
-        (e.g. "small", "middle", "large") and should not be changed.
     """
 
     TYPE_HEADERS = ('discrete', 'd', 'categorical')
 
     presorted_values = []
 
-    def __init__(self, name="", values=(), ordered=False, compute_value=None,
-                 *, sparse=False):
+    def __init__(
+            self, name="", values=(), compute_value=None, *, sparse=False
+    ):
         """ Construct a discrete variable descriptor with the given values. """
-        self.values = list(values)
-        if not all(isinstance(value, str) for value in self.values):
+        values = tuple(values)  # some people (including me) pass a generator
+        if not all(isinstance(value, str) for value in values):
             raise TypeError("values of DiscreteVariables must be strings")
+
         super().__init__(name, compute_value, sparse=sparse)
-        self.ordered = ordered
+        self._values = values
+        self._value_index = {value: i for i, value in enumerate(values)}
+
+    @property
+    def values(self):
+        return self._values
 
     def get_mapping_from(self, other):
         return np.array(
-            [self.values.index(value) if value in self.values else np.nan
-             for value in other.values], dtype=float)
+            [self._value_index.get(value, np.nan) for value in other.values],
+            dtype=float)
 
     def get_mapper_from(self, other):
         mapping = self.get_mapping_from(other)
@@ -618,7 +671,7 @@ class DiscreteVariable(Variable):
                     value.data[nans] = 0
                     value.data[col] = mapping[value.data[col].astype(int)]
                     value.data[nans] = np.nan
-                    return
+                    return None
 
                 # Dense and CSC map a contiguous area
                 if isinstance(value, np.ndarray) and value.ndim == 2:
@@ -635,11 +688,11 @@ class DiscreteVariable(Variable):
                 col[nans] = 0
                 col[:] = mapping[col.astype(int)]
                 col[nans] = np.nan
-                return
+                return None
 
             # Mapping into a copy
             if isinstance(value, (int, float)):
-                return mapping[int(value)] if value == value else value
+                return value if np.isnan(value) else mapping[int(value)]
             if isinstance(value, str):
                 return mapping[other.values.index(value)]
             if isinstance(value, np.ndarray):
@@ -669,7 +722,7 @@ class DiscreteVariable(Variable):
                 value.data = mapper(value.data)
                 return value
             if isinstance(value, Iterable):
-                return type(value)(mapping[int(val)] if val == val else val
+                return type(value)(val if np.isnan(val) else mapping[int(val)]
                                    for val in value)
             raise ValueError(
                 f"invalid type for value(s): {type(value).__name__}")
@@ -700,14 +753,19 @@ class DiscreteVariable(Variable):
         if not isinstance(s, str):
             raise TypeError('Cannot convert {} to value of "{}"'.format(
                 type(s).__name__, self.name))
-        return self.values.index(s)
+        if s not in self._value_index:
+            raise ValueError(f"Value {s} does not exist")
+        return self._value_index[s]
 
     def add_value(self, s):
         """ Add a value `s` to the list of values.
         """
         if not isinstance(s, str):
             raise TypeError("values of DiscreteVariables must be strings")
-        self.values.append(s)
+        if s in self._value_index:
+            return
+        self._value_index[s] = len(self.values)
+        self._values += (s, )
 
     def val_from_str_add(self, s):
         """
@@ -719,12 +777,13 @@ class DiscreteVariable(Variable):
         :rtype: float
         """
         s = str(s) if s is not None else s
-        try:
-            return ValueUnknown if s in self.unknown_str \
-                else self.values.index(s)
-        except ValueError:
+        if s in self.unknown_str:
+            return ValueUnknown
+        val = self._value_index.get(s)
+        if val is None:
             self.add_value(s)
-            return len(self.values) - 1
+            val = len(self.values) - 1
+        return val
 
     def repr_val(self, val):
         """
@@ -745,17 +804,21 @@ class DiscreteVariable(Variable):
         if not self.name:
             raise PickleError("Variables without names cannot be pickled")
         __dict__ = dict(self.__dict__)
-        __dict__.pop("values")
-        return make_variable, (self.__class__, self._compute_value, self.name,
-                               self.values, self.ordered), \
+        __dict__.pop("_values")
+        return (
+            make_variable,
+            (self.__class__, self._compute_value, self.name, self.values),
             __dict__
+        )
 
-    def copy(self, compute_value=None, *, name=None, values=None, **_):
+    def copy(self, compute_value=Variable._CopyComputeValue,
+             *, name=None, values=None, **_):
+        # pylint: disable=arguments-differ
         if values is not None and len(values) != len(self.values):
             raise ValueError(
                 "number of values must match the number of original values")
         return super().copy(compute_value=compute_value, name=name,
-                            values=values or self.values, ordered=self.ordered)
+                            values=values or self.values)
 
 
 class StringVariable(Variable):
@@ -782,10 +845,10 @@ class StringVariable(Variable):
     @staticmethod
     def str_val(val):
         """Return a string representation of the value."""
-        if val is "":
+        if isinstance(val, str) and val == "":
             return "?"
         if isinstance(val, Value):
-            if val.value is "":
+            if not val.value:
                 return "?"
             val = val.value
         return str(val)
@@ -807,12 +870,12 @@ class TimeVariable(ContinuousVariable):
 
     If time is specified without a date, Unix epoch is assumed.
 
-    If time is specified wihout an UTC offset, localtime is assumed.
+    If time is specified without an UTC offset, localtime is assumed.
     """
     _all_vars = {}
     TYPE_HEADERS = ('time', 't')
     UNIX_EPOCH = datetime(1970, 1, 1)
-    _ISO_FORMATS = [
+    _ISO_FORMATS = (
         # have_date, have_time, format_str
         # in order of decreased probability
         (1, 1, '%Y-%m-%d %H:%M:%S%z'),
@@ -847,7 +910,10 @@ class TimeVariable(ContinuousVariable):
         # so these two lines must be in this order
         (1, 0, '%Y-%m'),
         (1, 0, '%Y-%j'),
-    ]
+    )
+    # Order in which `_ISO_FORMATS` are tried. Must never change order of
+    # last 2 items. Only modified via assignment in `parse`.
+    __ISO_FORMATS_PROBE_SEQ = list(range(len(_ISO_FORMATS)))
     # The regex that matches all above formats
     REGEX = (r'^('
              r'\d{1,4}-\d{2}-\d{2}([ T]\d{2}:\d{2}(:\d{2}(\.\d+)?([+-]\d{4})?)?)?|'
@@ -857,26 +923,116 @@ class TimeVariable(ContinuousVariable):
              r'\d{1,4}(-?\d{2,3})?'
              r')$')
 
+    ADDITIONAL_FORMATS = {
+        "2021-11-25": (("%Y-%m-%d",), 1, 0),
+        "25.11.2021": (("%d.%m.%Y", "%d. %m. %Y"), 1, 0),
+        "25.11.21": (("%d.%m.%y", "%d. %m. %y"), 1, 0),
+        "11/25/2021": (("%m/%d/%Y",), 1, 0),
+        "11/25/21": (("%m/%d/%y",), 1, 0),
+        "20211125": (("%Y%m%d",), 1, 0),
+        # it would be too many options if we also include all time formats with
+        # with lengths up to minutes, up to seconds and up to milliseconds,
+        # joining all tree options under 00:00:00
+        "2021-11-25 00:00:00": (
+            (
+                "%Y-%m-%d %H:%M",
+                "%Y-%m-%d %H:%M:%S",
+                "%Y-%m-%d %H:%M:%S.%f",
+            ),
+            1,
+            1,
+        ),
+        "25.11.2021 00:00:00": (
+            (
+                "%d.%m.%Y %H:%M",
+                "%d. %m. %Y %H:%M",
+                "%d.%m.%Y %H:%M:%S",
+                "%d. %m. %Y %H:%M:%S",
+                "%d.%m.%Y %H:%M:%S.%f",
+                "%d. %m. %Y %H:%M:%S.%f",
+            ),
+            1,
+            1,
+        ),
+        "25.11.21 00:00:00": (
+            (
+                "%d.%m.%y %H:%M",
+                "%d. %m. %y %H:%M",
+                "%d.%m.%y %H:%M:%S",
+                "%d. %m. %y %H:%M:%S",
+                "%d.%m.%y %H:%M:%S.%f",
+                "%d. %m. %y %H:%M:%S.%f",
+            ),
+            1,
+            1,
+        ),
+        "11/25/2021 00:00:00": (
+            (
+                "%m/%d/%Y %H:%M",
+                "%m/%d/%Y %H:%M:%S",
+                "%m/%d/%Y %H:%M:%S.%f",
+            ),
+            1,
+            1,
+        ),
+        "11/25/21 00:00:00": (
+            (
+                "%m/%d/%y %H:%M",
+                "%m/%d/%y %H:%M:%S",
+                "%m/%d/%y %H:%M:%S.%f",
+            ),
+            1,
+            1,
+        ),
+        "20211125000000": (("%Y%m%d%H%M", "%Y%m%d%H%M%S", "%Y%m%d%H%M%S.%f"), 1, 1),
+        "00:00:00": (("%H:%M", "%H:%M:%S", "%H:%M:%S.%f"), 0, 1),
+        "000000": (("%H%M", "%H%M%S", "%H%M%S.%f"), 0, 1),
+        "2021": (("%Y",), 1, 0),
+        "11-25": (("%m-%d",), 1, 0),
+        "25.11.": (("%d.%m.", "%d. %m."), 1, 0),
+        "11/25": (("%m/%d",), 1, 0),
+        "1125": (("%m%d",), 1, 0),
+    }
+
     class InvalidDateTimeFormatError(ValueError):
         def __init__(self, date_string):
             super().__init__(
-                "Invalid datetime format '{}'. "
-                "Only ISO 8601 supported.".format(date_string))
+                f"Invalid datetime format '{date_string}'. Only ISO 8601 supported."
+            )
 
     _matches_iso_format = re.compile(REGEX).match
 
-    # UTC offset and associated timezone. If parsed datetime values provide an
-    # offset, it is used for display. If not all values have the same offset,
-    # +0000 (=UTC) timezone is used and utc_offset is set to False.
-    utc_offset = None
-    timezone = timezone.utc
+    # If parsed datetime values provide an offset or timzone, it is used for display.
+    # If not all values have the same offset, +0000 (=UTC) timezone is used
+    _timezone = None
 
     def __init__(self, *args, have_date=0, have_time=0, **kwargs):
         super().__init__(*args, **kwargs)
         self.have_date = have_date
         self.have_time = have_time
 
-    def copy(self, compute_value=None, *, name=None, **_):
+    @property
+    def timezone(self):
+        if self._timezone is None or self._timezone == "different timezones":
+            return timezone.utc
+        else:
+            return self._timezone
+
+    @timezone.setter
+    def timezone(self, tz):
+        """
+        Set timezone value:
+        - if self._timezone is None set it to new timezone
+        - if current timezone is different that new indicate that TimeVariable
+          have two date-times with different timezones
+        - if timezones are same keep it
+        """
+        if self._timezone is None:
+            self._timezone = tz
+        elif tz != self.timezone:
+            self._timezone = "different timezones"
+
+    def copy(self, compute_value=Variable._CopyComputeValue, *, name=None, **_):
         return super().copy(compute_value=compute_value, name=name,
                             have_date=self.have_date, have_time=self.have_time)
 
@@ -926,7 +1082,9 @@ class TimeVariable(ContinuousVariable):
         """
         if datestr in MISSING_VALUES:
             return Unknown
+
         datestr = datestr.strip().rstrip('Z')
+        datestr = self._tzre_sub(datestr)
 
         if not self._matches_iso_format(datestr):
             try:
@@ -937,17 +1095,20 @@ class TimeVariable(ContinuousVariable):
             except ValueError:
                 raise self.InvalidDateTimeFormatError(datestr)
 
-        for i, (have_date, have_time, fmt) in enumerate(self._ISO_FORMATS):
+        try_order = self.__ISO_FORMATS_PROBE_SEQ
+        for i, (have_date, have_time, fmt) in enumerate(
+                map(self._ISO_FORMATS.__getitem__, try_order)):
             try:
                 dt = datetime.strptime(datestr, fmt)
             except ValueError:
                 continue
             else:
-                # Pop this most-recently-used format to front
-                if 0 < i < len(self._ISO_FORMATS) - 2:
-                    self._ISO_FORMATS[i], self._ISO_FORMATS[0] = \
-                        self._ISO_FORMATS[0], self._ISO_FORMATS[i]
-
+                # Pop this most-recently-used format index to front,
+                # excluding last 2
+                if 0 < i < len(try_order) - 2:
+                    try_order = try_order.copy()
+                    try_order[i], try_order[0] = try_order[0], try_order[i]
+                    TimeVariable.__ISO_FORMATS_PROBE_SEQ = try_order
                 self.have_date |= have_date
                 self.have_time |= have_time
                 if not have_date:
@@ -958,16 +1119,8 @@ class TimeVariable(ContinuousVariable):
         else:
             raise self.InvalidDateTimeFormatError(datestr)
 
-        # Remember UTC offset. If not all parsed values share the same offset,
-        # remember none of it.
         offset = dt.utcoffset()
-        if self.utc_offset is not False:
-            if offset and self.utc_offset is None:
-                self.utc_offset = offset
-                self.timezone = timezone(offset)
-            elif self.utc_offset != offset:
-                self.utc_offset = False
-                self.timezone = timezone.utc
+        self.timezone = timezone(offset) if offset is not None else None
 
         # Convert time to UTC timezone. In dates without timezone,
         # localtime is assumed. See also:
