@@ -12,7 +12,7 @@ from Orange.data import DiscreteVariable, Domain
 from Orange.data.sql.table import SqlTable
 from Orange.statistics import distribution, contingency, util as ut
 from Orange.statistics.basic_stats import BasicStats
-from Orange.util import Reprable
+from Orange.util import Reprable, utc_from_timestamp
 from .transformation import Transformation
 from . import _discretize
 
@@ -83,6 +83,12 @@ class Discretizer(Transformation):
         dvar.to_sql = to_sql
         return dvar
 
+    def __eq__(self, other):
+        return super().__eq__(other) and self.points == other.points
+
+    def __hash__(self):
+        return hash((type(self), self.variable, tuple(self.points)))
+
 
 class BinSql:
     def __init__(self, var, points):
@@ -141,6 +147,8 @@ class EqualFreq(Discretization):
         else:
             d = distribution.get_distribution(data, attribute)
             points = _discretize.split_eq_freq(d, self.n)
+            # np.unique handles cases in which differences are below precision
+            points = list(np.unique(points))
         return Discretizer.create_discretized_var(
             data.domain[attribute], points)
 
@@ -166,8 +174,11 @@ class EqualWidth(Discretization):
             else:
                 values = data[:, attribute]
                 values = values.X if values.X.size else values.Y
-                min, max = ut.nanmin(values), ut.nanmax(values)
-                points = self._split_eq_width(min, max)
+                if values.size:
+                    min, max = ut.nanmin(values), ut.nanmax(values)
+                    points = self._split_eq_width(min, max)
+                else:
+                    points = []
         return Discretizer.create_discretized_var(
             data.domain[attribute], points)
 
@@ -309,8 +320,11 @@ def decimal_binnings(
         nbins = np.round((mx_ - mn_) / width)
         if min_bins <= nbins <= max_bins \
                 and (not bins or bins[-1].nbins != nbins):
-            bin_def = BinDefinition(mn_ + width * np.arange(nbins + 1),
-                                    label_fmt, None, width)
+            bins_ = mn_ + width * np.arange(nbins + 1)
+            # to prevent values on the edge of the bin fall in the wrong bin
+            # due to precision error on decimals that are not precise
+            bins_ = np.around(bins_, decimals=np.finfo(bins_.dtype).precision)
+            bin_def = BinDefinition(bins_, label_fmt, None, width)
             bins.append(bin_def)
     return bins
 
@@ -348,7 +362,8 @@ def time_binnings(data, *, min_bins=2, max_bins=50, min_unique=5, add_unique=0):
             (number_of_seconds_since_epoch, label).
     """
     mn, mx, unique = _min_max_unique(data)
-    mn, mx = time.gmtime(mn), time.gmtime(mx)
+    mn = utc_from_timestamp(mn).timetuple()
+    mx = utc_from_timestamp(mx).timetuple()
     bins = []
     if len(unique) <= max(min_unique, add_unique):
         bins.append(_unique_time_bins(unique))
@@ -358,21 +373,20 @@ def time_binnings(data, *, min_bins=2, max_bins=50, min_unique=5, add_unique=0):
 
 
 def _time_binnings(mn, mx, min_pts, max_pts):
-    yfmt = "%y " if mn.tm_year >= 1950 else "%Y "
     bins = []
     for place, step, fmt, unit in (
             [(5, x, "%H:%M:%S", "second") for x in (1, 5, 10, 15, 30)] +
             [(4, x, "%b %d %H:%M", "minute") for x in (1, 5, 10, 15, 30)] +
-            [(3, x, yfmt + "%b %d %H:%M", "hour") for x in (1, 2, 3, 6, 12)] +
-            [(2, 1, yfmt + "%b %d", "day")] +
-            [(2, x, yfmt + "%b %d", "week") for x in (7, 14)] +
-            [(1, x, yfmt + "%b", "month") for x in (1, 2, 3, 6)] +
-            [(0, x, yfmt.strip(), "year") for x in (1, 2, 5, 10, 25, 50, 100)]):
+            [(3, x, "%y %b %d %H:%M", "hour") for x in (1, 2, 3, 6, 12)] +
+            [(2, 1, "%y %b %d", "day")] +
+            [(2, x, "%y %b %d", "week") for x in (7, 14)] +
+            [(1, x, "%y %b", "month") for x in (1, 2, 3, 6)] +
+            [(0, x, "%Y", "year") for x in (1, 2, 5, 10, 25, 50, 100)]):
         times = _time_range(mn, mx, place, step, min_pts, max_pts)
         if not times:
             continue
         times = [time.struct_time(t + (0, 0, 0)) for t in times]
-        thresholds = [calendar.timegm(t) for t in times]
+        thresholds = np.array([calendar.timegm(t) for t in times])
         labels = [time.strftime(fmt, t) for t in times]
         short_labels = _simplified_labels(labels)
         if place == 2 and step >= 7:
@@ -437,10 +451,10 @@ def _simplified_labels(labels):
         if len(firsts) > 1:
             break
         to_remove = firsts.pop()
-        flen = len(to_remove) + 1
+        flen = len(to_remove)
         if any(len(lab) == flen for lab in labels):
             break
-        labels = [lab[flen:] for lab in labels]
+        labels = [lab[flen+1:] for lab in labels]
     for i in range(len(labels) - 1, 0, -1):
         for k, c, d in zip(count(), labels[i].split(), labels[i - 1].split()):
             if c != d:
@@ -454,8 +468,8 @@ def _simplified_labels(labels):
 
 
 def _unique_time_bins(unique):
-    times = [time.gmtime(x) for x in unique]
-    fmt = f'{"%y " if times[0][0] >= 1950 else "%Y "} %b %d'
+    times = [utc_from_timestamp(x).timetuple() for x in unique]
+    fmt = f'%y %b %d'
     fmt += " %H:%M" * (len({t[2:] for t in times}) > 1)
     fmt += ":%S" * bool(np.all(unique % 60 == 0))
     labels = [time.strftime(fmt, x) for x in times]
@@ -673,7 +687,7 @@ class EntropyMDL(Discretization):
         delta = np.log2(3 ** k - 2) - (k * ES - k1 * ES1 - k2 * ES2)
         N = float(np.sum(S_c))
 
-        if Gain > np.log2(N - 1) / N + delta / N:
+        if N > 1 and Gain > np.log2(N - 1) / N + delta / N:
             # Accept the cut point and recursively split the subsets.
             left, right = [], []
             if k1 > 1 and cut_index > 1:
