@@ -15,7 +15,7 @@ import numpy as np
 import scipy.stats as ss
 import scipy.sparse as sp
 from AnyQt.QtCore import Qt, QSize, QRectF, QModelIndex, pyqtSlot, \
-    QRegExp, QItemSelection, QItemSelectionRange, QItemSelectionModel
+    QItemSelection, QItemSelectionRange, QItemSelectionModel
 from AnyQt.QtGui import QPainter, QColor
 from AnyQt.QtWidgets import QStyledItemDelegate, QGraphicsScene, QTableView, \
     QHeaderView, QStyle, QStyleOptionViewItem
@@ -23,6 +23,7 @@ from AnyQt.QtWidgets import QStyledItemDelegate, QGraphicsScene, QTableView, \
 import Orange.statistics.util as ut
 from Orange.data import Table, StringVariable, DiscreteVariable, \
     ContinuousVariable, TimeVariable, Domain, Variable
+from Orange.util import utc_from_timestamp
 from Orange.widgets import widget, gui
 from Orange.widgets.data.utils.histogram import Histogram
 from Orange.widgets.settings import Setting, ContextSetting, \
@@ -30,7 +31,6 @@ from Orange.widgets.settings import Setting, ContextSetting, \
 from Orange.widgets.utils.itemmodels import DomainModel, AbstractSortTableModel
 from Orange.widgets.utils.signals import Input, Output
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
 
 
 def _categorical_entropy(x):
@@ -69,8 +69,8 @@ def format_time_diff(start, end, round_up_after=2):
     str
 
     """
-    start = datetime.datetime.fromtimestamp(start)
-    end = datetime.datetime.fromtimestamp(end)
+    start = utc_from_timestamp(start)
+    end = utc_from_timestamp(end)
     diff = abs(end - start)  # type: datetime.timedelta
 
     # Get the different resolutions
@@ -110,7 +110,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
     HIDDEN_VAR_TYPES = (StringVariable,)
 
     class Columns(IntEnum):
-        ICON, NAME, DISTRIBUTION, CENTER, DISPERSION, MIN, MAX, MISSING = range(8)
+        ICON, NAME, DISTRIBUTION, CENTER, MEDIAN, DISPERSION, MIN, MAX, \
+            MISSING = range(9)
 
         @property
         def name(self):
@@ -118,9 +119,10 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
                     self.NAME: '名称',
                     self.DISTRIBUTION: '分布',
                     self.CENTER: '集中趋势',
+                    self.MEDIAN: '中位数',
                     self.DISPERSION: '离散',
-                    self.MIN: '最小.',
-                    self.MAX: '最大.',
+                    self.MIN: '最小',
+                    self.MAX: '最大',
                     self.MISSING: '缺失',
                     }[self.value]
 
@@ -157,7 +159,7 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
         no_data = np.array([])
         self._variable_types = self._variable_names = no_data
-        self._min = self._max = self._center = no_data
+        self._min = self._max = self._center = self._median = no_data
         self._dispersion = no_data
         self._missing = no_data
         # Clear model initially to set default values
@@ -175,11 +177,12 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         self.domain = domain = data.domain
         self.target_var = None
 
-        self.__attributes = self.__filter_attributes(domain.attributes, self.table.X)
-        # We disable pylint warning because the `Y` property squeezes vectors,
-        # while we need a 2d array, which `_Y` provides
-        self.__class_vars = self.__filter_attributes(domain.class_vars, self.table._Y)  # pylint: disable=protected-access
-        self.__metas = self.__filter_attributes(domain.metas, self.table.metas)
+        self.__attributes = self.__filter_attributes(
+            domain.attributes, self.table.X)
+        self.__class_vars = self.__filter_attributes(
+            domain.class_vars, self.table.Y.reshape((len(self.table.Y), -1)))
+        self.__metas = self.__filter_attributes(
+            domain.metas, self.table.metas)
         self.__attributes_set = set(self.__metas[0])
         self.__class_vars_set = set(self.__class_vars[0])
         self.__metas_set = set(self.__metas[0])
@@ -205,7 +208,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
     @property
     def variables(self):
-        matrices = [self.__attributes[0], self.__class_vars[0], self.__metas[0]]
+        matrices = [self.__attributes[0],
+                    self.__class_vars[0], self.__metas[0]]
         if not any(m.size for m in matrices):
             return []
         return np.hstack(matrices)
@@ -214,12 +218,15 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
     def _attr_indices(attrs):
         # type: (List) -> Tuple[List[int], List[int], List[int], List[int]]
         """Get the indices of different attribute types eg. discrete."""
-        disc_var_idx = [i for i, attr in enumerate(attrs) if isinstance(attr, DiscreteVariable)]
+        disc_var_idx = [i for i, attr in enumerate(
+            attrs) if isinstance(attr, DiscreteVariable)]
         cont_var_idx = [i for i, attr in enumerate(attrs)
                         if isinstance(attr, ContinuousVariable)
                         and not isinstance(attr, TimeVariable)]
-        time_var_idx = [i for i, attr in enumerate(attrs) if isinstance(attr, TimeVariable)]
-        string_var_idx = [i for i, attr in enumerate(attrs) if isinstance(attr, StringVariable)]
+        time_var_idx = [i for i, attr in enumerate(
+            attrs) if isinstance(attr, TimeVariable)]
+        string_var_idx = [i for i, attr in enumerate(
+            attrs) if isinstance(attr, StringVariable)]
         return disc_var_idx, cont_var_idx, time_var_idx, string_var_idx
 
     def __filter_attributes(self, attributes, matrix):
@@ -237,7 +244,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         matrices = list(filter(lambda tup: tup[1].size, matrices))
 
         self._variable_types = np.array([type(var) for var in self.variables])
-        self._variable_names = np.array([var.name.lower() for var in self.variables])
+        self._variable_names = np.array(
+            [var.name.lower() for var in self.variables])
         self._min = self.__compute_stat(
             matrices,
             discrete_f=lambda x: ut.nanmin(x, axis=0),
@@ -271,15 +279,23 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             if sp.issparse(x):
                 x = x.todense(order="C")
             # return ss.mode(x, *args, **kwargs)[0]
-            return ut.nanmode(x, *args, **kwargs)[0]  # Temporary replacement for scipy
+            # Temporary replacement for scipy
+            return ut.nanmode(x, *args, **kwargs)[0]
 
         self._center = self.__compute_stat(
             matrices,
-            discrete_f=lambda x: __mode(x, axis=0),
+            discrete_f=None,
             continuous_f=lambda x: ut.nanmean(x, axis=0),
             time_f=lambda x: ut.nanmean(x, axis=0),
         )
 
+        self._median = self.__compute_stat(
+            matrices,
+            discrete_f=lambda x: __mode(x, axis=0),
+            continuous_f=lambda x: ut.nanmedian(x, axis=0),
+            time_f=lambda x: ut.nanmedian(x, axis=0),
+        )
+
     def get_statistics_matrix(self, variables=None, return_labels=False):
         """Get the numeric computed statistics in a single matrix. Optionally,
         we can specify for which variables we want the stats. Also, we can get
@@ -311,14 +327,16 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             indices = ...
 
         matrix = np.vstack((
-            self._center[indices], self._dispersion[indices],
+            self._center[indices], self._median[indices],
+            self._dispersion[indices],
             self._min[indices], self._max[indices], self._missing[indices],
         )).T
 
         # Return string labels for the returned matrix columns e.g. 'Mean',
         # 'Dispersion' if requested
         if return_labels:
-            labels = [self.Columns.CENTER.name, self.Columns.DISPERSION.name,
+            labels = [self.Columns.CENTER.name, self.Columns.MEDIAN.name,
+                      self.Columns.DISPERSION.name,
                       self.Columns.MIN.name, self.Columns.MAX.name,
                       self.Columns.MISSING.name]
             return labels, matrix
@@ -375,14 +393,17 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
         to group those variable types together.
         """
         # Prepare indices for variable types so we can group them together
-        order = [DiscreteVariable, ContinuousVariable, TimeVariable, StringVariable]
+        order = [ContinuousVariable, TimeVariable,
+                 DiscreteVariable, StringVariable]
         mapping = {var: idx for idx, var in enumerate(order)}
         vmapping = np.vectorize(mapping.__getitem__)
         var_types_indices = vmapping(self._variable_types)
 
         # Store the variable name sorted indices so we can pass a default
         # order when sorting by multiple keys
-        var_name_indices = np.argsort(self._variable_names)
+        # Double argsort is "inverse" argsort:
+        # data will be *sorted* by these indices
+        var_name_indices = np.argsort(np.argsort(self._variable_names))
 
         # Prepare vartype indices so ready when needed
         disc_idx, _, time_idx, str_idx = self._attr_indices(self.variables)
@@ -406,6 +427,13 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             vals[disc_idx] = var_name_indices[disc_idx]
             vals[str_idx] = var_name_indices[str_idx]
             return np.vstack((var_types_indices, np.zeros_like(vals), vals)).T
+        # Sort by: (type, median)
+        elif column == self.Columns.MEDIAN:
+            # Sorting discrete or string values by median makes no sense
+            vals = np.array(self._median)
+            vals[disc_idx] = var_name_indices[disc_idx]
+            vals[str_idx] = var_name_indices[str_idx]
+            return np.vstack((var_types_indices, np.zeros_like(vals), vals)).T
         # Sort by: (type, dispersion)
         elif column == self.Columns.DISPERSION:
             # Sort time variables by their dispersion, which is not stored in
@@ -443,12 +471,21 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
 
     def _argsortData(self, data, order):
         if data.ndim == 1:
-            indices = np.argsort(data, kind='mergesort')
-            if order == Qt.DescendingOrder:
-                indices = indices[::-1]
-            # Always sort NaNs last
             if np.issubdtype(data.dtype, np.number):
-                indices = np.roll(indices, -np.isnan(data).sum())
+                if order == Qt.DescendingOrder:
+                    data = -data
+                indices = np.argsort(data, kind='stable')
+                # Always sort NaNs last
+                if np.issubdtype(data.dtype, np.number):
+                    indices = np.roll(indices, -np.isnan(data).sum())
+            else:
+                # When not sorting by numbers, we can't do data = -data, but
+                # use indices = indices[::-1] instead. This is not stable, but
+                # doesn't matter because we use this only for variable names
+                # which are guaranteed to be unique
+                indices = np.argsort(data)
+                if order == Qt.DescendingOrder:
+                    indices = indices[::-1]
         else:
             assert np.issubdtype(data.dtype, np.number), \
                 'We do not deal with non numeric values in sorting by ' \
@@ -503,7 +540,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
             def format_zeros(str_val):
                 """Zeros should be handled separately as they cannot be negative."""
                 if float(str_val) == 0:
-                    num_decimals = min(self.variables[row].number_of_decimals, 2)
+                    num_decimals = min(
+                        self.variables[row].number_of_decimals, 2)
                     str_val = f"{0:.{num_decimals}f}"
                 return str_val
 
@@ -531,6 +569,7 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
                             variable=attribute,
                             color_attribute=self.target_var,
                             border=(0, 0, 2, 0),
+                            bottom_padding=4,
                             border_color='#ccc',
                         )
                         scene.addItem(histogram)
@@ -538,6 +577,8 @@ class FeatureStatisticsTableModel(AbstractSortTableModel):
                     return self.__distributions_cache[row]
             elif column == self.Columns.CENTER:
                 return render_value(self._center[row])
+            elif column == self.Columns.MEDIAN:
+                return render_value(self._median[row])
             elif column == self.Columns.DISPERSION:
                 if isinstance(attribute, TimeVariable):
                     return format_time_diff(self._min[row], self._max[row])
@@ -624,8 +665,10 @@ class FeatureStatisticsTableView(QTableView):
         hheader.setSectionResizeMode(QHeaderView.Interactive)
 
         columns = model.Columns
-        hheader.setSectionResizeMode(columns.ICON.index, QHeaderView.ResizeToContents)
-        hheader.setSectionResizeMode(columns.DISTRIBUTION.index, QHeaderView.Stretch)
+        hheader.setSectionResizeMode(
+            columns.ICON.index, QHeaderView.ResizeToContents)
+        hheader.setSectionResizeMode(
+            columns.DISTRIBUTION.index, QHeaderView.Stretch)
 
         vheader = self.verticalHeader()
         vheader.setVisible(False)
@@ -647,7 +690,7 @@ class FeatureStatisticsTableView(QTableView):
         if logical_index is not self.model().Columns.DISTRIBUTION.index:
             return
         ratio_width, ratio_height = self.HISTOGRAM_ASPECT_RATIO
-        unit_width = new_size / ratio_width
+        unit_width = new_size // ratio_width
         new_height = unit_width * ratio_height
         effective_height = max(new_height, self.MINIMUM_HISTOGRAM_HEIGHT)
         effective_height = min(effective_height, self.MAXIMUM_HISTOGRAM_HEIGHT)
@@ -664,7 +707,8 @@ class FeatureStatisticsTableView(QTableView):
         top_row = self.indexAt(self.rect().topLeft()).row()
         bottom_row = self.indexAt(self.rect().bottomLeft()).row()
         middle_row = top_row + (bottom_row - top_row) // 2
-        self.scrollTo(self.model().index(middle_row, 0), QTableView.PositionAtCenter)
+        self.scrollTo(self.model().index(middle_row, 0),
+                      QTableView.PositionAtCenter)
 
 
 class NoFocusRectDelegate(QStyledItemDelegate):
@@ -676,7 +720,7 @@ class NoFocusRectDelegate(QStyledItemDelegate):
         super().paint(painter, option, index)
 
 
-class DistributionDelegate(QStyledItemDelegate):
+class DistributionDelegate(NoFocusRectDelegate):
     def paint(self, painter, option, index):
         # type: (QPainter, QStyleOptionViewItem, QModelIndex) -> None
         scene = index.data(Qt.DisplayRole)  # type: Optional[QGraphicsScene]
@@ -684,15 +728,8 @@ class DistributionDelegate(QStyledItemDelegate):
             return super().paint(painter, option, index)
 
         painter.setRenderHint(QPainter.Antialiasing)
-
-        if option.state & QStyle.State_Selected:
-            background_color = option.palette.highlight()
-        else:
-            background_color = index.data(Qt.BackgroundRole)
-        if background_color is not None:
-            scene.setBackgroundBrush(background_color)
-
-        scene.render(painter, target=QRectF(option.rect), mode=Qt.IgnoreAspectRatio)
+        scene.render(painter, target=QRectF(
+            option.rect), mode=Qt.IgnoreAspectRatio)
 
         # pylint complains about inconsistent return statements
         return None
@@ -701,19 +738,19 @@ class DistributionDelegate(QStyledItemDelegate):
 class OWFeatureStatistics(widget.OWWidget):
     name = '特征统计(Feature Statistics)'
     description = '显示数据特征的基本统计信息。'
-    icon = 'icons/FeatureStatistics.svg'
-    category = "Data"
+    category = "数据(Data)"
     keywords = ['tezheng', 'tongji', 'tezhengtongji']
+    icon = 'icons/FeatureStatistics.svg'
 
     class Inputs:
         data = Input('数据(Data)', Table, default=True, replaces=['Data'])
 
     class Outputs:
-        reduced_data = Output('选中的数据(Reduced Data)', Table, default=True, replaces=['Reduced Data'])
+        reduced_data = Output('选中的数据(Reduced Data)', Table,
+                              default=True, replaces=['Reduced Data'])
         statistics = Output('统计(Statistics)', Table, replaces=['Statistics'])
 
-    want_control_area = False
-    buttons_area_orientation = Qt.Vertical
+    want_main_area = False
 
     settingsHandler = DomainContextHandler()
     settings_version = 2
@@ -722,7 +759,7 @@ class OWFeatureStatistics(widget.OWWidget):
     color_var = ContextSetting(None)  # type: Optional[Variable]
     # filter_string = ContextSetting('')
 
-    sorting = Setting((0, Qt.DescendingOrder))
+    sorting = Setting((0, Qt.AscendingOrder))
     selected_vars = ContextSetting([], schema_only=True)
 
     def __init__(self):
@@ -730,60 +767,32 @@ class OWFeatureStatistics(widget.OWWidget):
 
         self.data = None  # type: Optional[Table]
 
-        # TODO: Implement filtering on the model
-        # filter_box = gui.vBox(self.controlArea, 'Filter')
-        # self.filter_text = gui.lineEdit(
-        #     filter_box, self, value='filter_string',
-        #     placeholderText='Filter variables by name',
-        #     callback=self._filter_table_variables, callbackOnType=True,
-        # )
-        # shortcut = QShortcut(QKeySequence('Ctrl+f'), self, self.filter_text.setFocus)
-        # shortcut.setWhatsThis('Filter variables by name')
-
-        box = gui.hBox(None, box=False)
-        box.setContentsMargins(0, 0, 0, 0)
-
-        self.color_var_model = DomainModel(
-            valid_types=(ContinuousVariable, DiscreteVariable),
-            placeholder='无',
-        )
-        self.cb_color_var = gui.comboBox(
-            box, master=self, value='color_var', model=self.color_var_model,
-            label='颜色:', orientation=Qt.Horizontal, contentsLength=13,
-            searchable=True
-        )
-        self.cb_color_var.activated.connect(self.__color_var_changed)
-
-        gui.rubber(box)
-        gui.auto_send(box, self, "auto_commit", box=None)
-        self.mainArea.layout().addWidget(box)
-
-        self.info.set_input_summary(self.info.NoInput)
-        self.info.set_output_summary(self.info.NoOutput)
-
         # Main area
         self.model = FeatureStatisticsTableModel(parent=self)
         self.table_view = FeatureStatisticsTableView(self.model, parent=self)
         self.table_view.selectionModel().selectionChanged.connect(self.on_select)
         self.table_view.horizontalHeader().sectionClicked.connect(self.on_header_click)
 
-        self.mainArea.layout().addWidget(self.table_view)
+        self.controlArea.layout().addWidget(self.table_view)
+
+        self.color_var_model = DomainModel(
+            valid_types=(ContinuousVariable, DiscreteVariable),
+            placeholder='None',
+        )
+        self.cb_color_var = gui.comboBox(
+            self.buttonsArea, master=self, value='color_var', model=self.color_var_model,
+            label='Color:', orientation=Qt.Horizontal, contentsLength=13,
+            searchable=True
+        )
+        self.cb_color_var.activated.connect(self.__color_var_changed)
+
+        gui.rubber(self.buttonsArea)
+        gui.auto_send(self.buttonsArea, self, "auto_commit")
 
     @staticmethod
     def sizeHint():
         return QSize(1050, 500)
 
-    def _filter_table_variables(self):
-        regex = QRegExp(self.filter_string)
-        # If the user explicitly types different cases, we assume they know
-        # what they are searching for and account for letter case in filter
-        different_case = (
-            any(c.islower() for c in self.filter_string) and
-            any(c.isupper() for c in self.filter_string)
-        )
-        if not different_case:
-            regex.setCaseSensitivity(Qt.CaseInsensitive)
-
     @Inputs.data
     def set_data(self, data):
         # Clear outputs and reset widget state
@@ -797,14 +806,11 @@ class OWFeatureStatistics(widget.OWWidget):
         self.data = data
 
         if data is not None:
-            self.info.set_input_summary(len(data),
-                                        format_summary_details(data))
             self.color_var_model.set_domain(data.domain)
             self.color_var = None
             if self.data.domain.class_vars:
                 self.color_var = self.data.domain.class_vars[0]
         else:
-            self.info.set_input_summary(self.info.NoInput)
             self.color_var_model.set_domain(None)
             self.color_var = None
         self.model.set_data(data)
@@ -812,7 +818,6 @@ class OWFeatureStatistics(widget.OWWidget):
         self.openContext(self.data)
         self.__restore_selection()
         self.__restore_sorting()
-        # self._filter_table_variables()
         self.__color_var_changed()
 
         self.commit()
@@ -822,7 +827,8 @@ class OWFeatureStatistics(widget.OWWidget):
         selection_model = self.table_view.selectionModel()
         selection = QItemSelection()
         if self.selected_vars:
-            var_indices = {var: i for i, var in enumerate(self.model.variables)}
+            var_indices = {var: i for i,
+                           var in enumerate(self.model.variables)}
             selected_indices = [var_indices[var] for var in self.selected_vars]
             for row in self.model.mapFromSourceRows(selected_indices):
                 selection.append(QItemSelectionRange(
@@ -859,19 +865,17 @@ class OWFeatureStatistics(widget.OWWidget):
 
     def commit(self):
         if not self.selected_vars:
-            self.info.set_output_summary(self.info.NoOutput)
             self.Outputs.reduced_data.send(None)
             self.Outputs.statistics.send(None)
             return
 
         # Send a table with only selected columns to output
         variables = self.selected_vars
-        self.info.set_output_summary(len(self.data[:, variables]),
-                                     format_summary_details(self.data[:, variables]))
         self.Outputs.reduced_data.send(self.data[:, variables])
 
         # Send the statistics of the selected variables to ouput
-        labels, data = self.model.get_statistics_matrix(variables, return_labels=True)
+        labels, data = self.model.get_statistics_matrix(
+            variables, return_labels=True)
         var_names = np.atleast_2d([var.name for var in variables]).T
         domain = Domain(
             attributes=[ContinuousVariable(name) for name in labels],
