@@ -1,4 +1,3 @@
-from AnyQt.QtCore import Qt
 from AnyQt.QtWidgets import QFrame
 
 from Orange.data import Table
@@ -7,7 +6,6 @@ from Orange.widgets import gui, widget
 from Orange.widgets.settings import Setting
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Input, Output
 
 
@@ -15,8 +13,8 @@ class OWPurgeDomain(widget.OWWidget):
     name = "清除特征(Purge Domain)"
     description = "从数据集中删除冗余值和特征,对值进行排序。"
     icon = "icons/PurgeDomain.svg"
-    category = "Data"
-    keywords = ["remove", "delete", "unused", 'qinli', 'qinchu', 'tezheng']
+    category = "数据(Data)"
+    keywords = ["remove", "delete", "unused", 'qinli', 'qinchu']
 
     class Inputs:
         data = Input("数据(Data)", Table, replaces=['Data'])
@@ -36,7 +34,6 @@ class OWPurgeDomain(widget.OWWidget):
 
     want_main_area = False
     resizing_enabled = False
-    buttons_area_orientation = Qt.Vertical
 
     feature_options = (('sortValues', '分类特征值排序'),
                        ('removeValues', '删除未使用的特征值'),
@@ -75,56 +72,42 @@ class OWPurgeDomain(widget.OWWidget):
             frame = QFrame()
             frame.setFrameShape(QFrame.HLine)
             frame.setFrameShadow(QFrame.Sunken)
-            parent.layout().addSpacing(6)
             parent.layout().addWidget(frame)
-            parent.layout().addSpacing(6)
 
         boxAt = gui.vBox(self.controlArea, "特征")
-        for not_first, (value, label) in enumerate(self.feature_options):
-            if not_first:
-                gui.separator(boxAt, 2)
+        for value, label in self.feature_options:
             gui.checkBox(boxAt, self, value, label,
-                         callback=self.optionsChanged)
+                         callback=self.commit.deferred)
         add_line(boxAt)
         gui.label(boxAt, self,
-                  "排序了: %(resortedAttrs)s, "
-                  "减少了: %(reducedAttrs)s, 删除了: %(removedAttrs)s")
+                  "已排序: %(resortedAttrs)s, "
+                  "已减少: %(reducedAttrs)s, 已删除: %(removedAttrs)s")
 
-        boxAt = gui.vBox(self.controlArea, "类别(Classes)", addSpace=True)
-        for not_first, (value, label) in enumerate(self.class_options):
-            if not_first:
-                gui.separator(boxAt, 2)
+        boxAt = gui.vBox(self.controlArea, "类别")
+        for value, label in self.class_options:
             gui.checkBox(boxAt, self, value, label,
-                         callback=self.optionsChanged)
+                         callback=self.commit.deferred)
         add_line(boxAt)
         gui.label(boxAt, self,
-                  "排序了: %(resortedClasses)s,"
-                  "减少了: %(reducedClasses)s, 删除了: %(removedClasses)s")
+                  "已排序: %(resortedClasses)s,"
+                  "已减少: %(reducedClasses)s, 已删除: %(removedClasses)s")
 
-        boxAt = gui.vBox(self.controlArea, "元属性", addSpace=True)
-        for not_first, (value, label) in enumerate(self.meta_options):
-            if not_first:
-                gui.separator(boxAt, 2)
+        boxAt = gui.vBox(self.controlArea, "元属性")
+        for value, label in self.meta_options:
             gui.checkBox(boxAt, self, value, label,
-                         callback=self.optionsChanged)
+                         callback=self.commit.deferred)
         add_line(boxAt)
         gui.label(boxAt, self,
-                  "减少了: %(reducedMetas)s, 删除了: %(removedMetas)s")
+                  "已减少: %(reducedMetas)s, 已删除: %(removedMetas)s")
 
         gui.auto_send(self.buttonsArea, self, "autoSend")
-        gui.rubber(self.controlArea)
-
-        self.info.set_input_summary(self.info.NoInput)
-        self.info.set_output_summary(self.info.NoOutput)
 
     @Inputs.data
     @check_sql_input
     def setData(self, dataset):
         if dataset is not None:
             self.data = dataset
-            self.info.set_input_summary(len(dataset),
-                                        format_summary_details(dataset))
-            self.unconditional_commit()
+            self.commit.now()
         else:
             self.removedAttrs = "-"
             self.reducedAttrs = "-"
@@ -136,12 +119,8 @@ class OWPurgeDomain(widget.OWWidget):
             self.reducedMetas = "-"
             self.Outputs.data.send(None)
             self.data = None
-            self.info.set_input_summary(self.info.NoInput)
-            self.info.set_output_summary(self.info.NoOutput)
-
-    def optionsChanged(self):
-        self.commit()
 
+    @gui.deferred
     def commit(self):
         if self.data is None:
             return
@@ -170,8 +149,6 @@ class OWPurgeDomain(widget.OWWidget):
         self.removedMetas = meta_res['removed']
         self.reducedMetas = meta_res['reduced']
 
-        self.info.set_output_summary(len(cleaned),
-                                     format_summary_details(cleaned))
         self.Outputs.data.send(cleaned)
 
     def send_report(self):
