@@ -3,7 +3,8 @@ import threading
 import itertools
 import concurrent.futures
 
-from collections import OrderedDict, namedtuple
+from collections import namedtuple
+from typing import List, Optional
 
 from math import isnan
 
@@ -12,7 +13,7 @@ from scipy.sparse import issparse
 
 from AnyQt.QtWidgets import (
     QTableView, QHeaderView, QAbstractButton, QApplication, QStyleOptionHeader,
-    QStyle, QStylePainter, QStyledItemDelegate
+    QStyle, QStylePainter
 )
 from AnyQt.QtGui import QColor, QClipboard
 from AnyQt.QtCore import (
@@ -30,16 +31,17 @@ from Orange.statistics import basic_stats
 
 from Orange.widgets import gui
 from Orange.widgets.settings import Setting
+from Orange.widgets.utils.itemdelegates import TableDataDelegate
 from Orange.widgets.utils.itemselectionmodel import (
     BlockSelectionModel, ranges, selection_blocks
 )
 from Orange.widgets.utils.tableview import TableView, \
     table_selection_to_mime_data
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.widget import OWWidget, Input, Output
+from Orange.widgets.widget import OWWidget, MultiInput, Output
 from Orange.widgets.utils import datacaching
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
-                                                 ANNOTATED_DATA_SIGNAL_Chinese_NAME)
+                                                 ANNOTATED_DATA_SIGNAL_NAME)
 from Orange.widgets.utils.itemmodels import TableModel
 from Orange.widgets.utils.state_summary import format_summary_details
 
@@ -170,22 +172,30 @@ class DataTableView(gui.HScrollStepMixin, TableView):
     input_slot: TableSlot
 
 
+class TableBarItemDelegate(gui.TableBarItem, TableDataDelegate):
+    pass
+
+
 class OWDataTable(OWWidget):
     name = "数据表(Data Table)"
     description = "在电子表格中查看数据集。"
     icon = "icons/Table.svg"
     priority = 50
-    keywords = ['shujubiao', 'biao']
-    category = "Data"
+    keywords = ['shujubiao']
+    category = "数据(Data)"
 
     buttons_area_orientation = Qt.Vertical
 
     class Inputs:
-        data = Input("数据(Data)", Table, multiple=True, replaces=['Data'])
+        data = MultiInput("数据(Data)", Table,
+                          auto_summary=False, filter_none=True, replaces=['Selected Data'])
 
     class Outputs:
-        selected_data = Output("选定的数据(Selected Data)", Table, default=True, replaces=['Selected Data'])
-        annotated_data = Output(ANNOTATED_DATA_SIGNAL_Chinese_NAME, Table, replaces=['Data'])
+        selected_data = Output("选定的数据(Selected Data)",
+                               Table, default=True, replaces=['Selected Data'])
+        annotated_data = Output('数据(Data)', Table)
+
+    buttons_area_orientation = Qt.Vertical
 
     show_distributions = Setting(False)
     dist_color_RGB = Setting((220, 220, 220, 255))
@@ -197,11 +207,11 @@ class OWDataTable(OWWidget):
     selected_rows = Setting([], schema_only=True)
     selected_cols = Setting([], schema_only=True)
 
+    settings_version = 2
+
     def __init__(self):
         super().__init__()
-
-        self._inputs = OrderedDict()
-
+        self._inputs: List[TableSlot] = []
         self.__pending_selected_rows = self.selected_rows
         self.selected_rows = None
         self.__pending_selected_cols = self.selected_cols
@@ -209,11 +219,9 @@ class OWDataTable(OWWidget):
 
         self.dist_color = QColor(*self.dist_color_RGB)
 
-        info_box = gui.vBox(self.controlArea, "Info")
+        info_box = gui.vBox(self.controlArea, "信息")
         self.info_text = gui.widgetLabel(info_box)
         self._set_input_summary(None)
-        self._set_output_summary(None)
-        gui.separator(self.controlArea)
 
         box = gui.vBox(self.controlArea, "变量")
         self.c_show_attribute_labels = gui.checkBox(
@@ -234,10 +242,11 @@ class OWDataTable(OWWidget):
 
         gui.rubber(self.controlArea)
 
-        reset = gui.button(
-            None, self, "恢复原始顺序", callback=self.restore_order,
-            tooltip="Show rows in the original order", autoDefault=False)
-        self.buttonsArea.layout().insertWidget(0, reset)
+        gui.button(self.buttonsArea, self, "恢复原始顺序",
+                   callback=self.restore_order,
+                   tooltip="Show rows in the original order",
+                   autoDefault=False,
+                   attribute=Qt.WA_LayoutUsesWidgetRect)
         gui.auto_send(self.buttonsArea, self, "auto_commit")
 
         # GUI with tabs
@@ -251,78 +260,90 @@ class OWDataTable(OWWidget):
     def sizeHint():
         return QSize(800, 500)
 
+    def _create_table_view(self):
+        view = DataTableView()
+        view.setSortingEnabled(True)
+        view.setItemDelegate(TableDataDelegate(view))
+
+        if self.select_rows:
+            view.setSelectionBehavior(QTableView.SelectRows)
+
+        header = view.horizontalHeader()
+        header.setSectionsMovable(True)
+        header.setSectionsClickable(True)
+        header.setSortIndicatorShown(True)
+        header.setSortIndicator(-1, Qt.AscendingOrder)
+
+        # QHeaderView does not 'reset' the model sort column,
+        # because there is no guaranty (requirement) that the
+        # models understand the -1 sort column.
+        def sort_reset(index, order):
+            if view.model() is not None and index == -1:
+                view.model().sort(index, order)
+        header.sortIndicatorChanged.connect(sort_reset)
+        return view
+
     @Inputs.data
-    def set_dataset(self, data, tid=None):
+    def set_dataset(self, index: int, data: Table):
         """Set the input dataset."""
-        if data is not None:
-            datasetname = getattr(data, "name", "Data")
-            if tid in self._inputs:
-                # update existing input slot
-                slot = self._inputs[tid]
-                view = slot.view
-                # reset the (header) view state.
-                view.setModel(None)
-                view.horizontalHeader().setSortIndicator(-1, Qt.AscendingOrder)
-                assert self.tabs.indexOf(view) != -1
-                self.tabs.setTabText(self.tabs.indexOf(view), datasetname)
-            else:
-                view = DataTableView()
-                view.setSortingEnabled(True)
+        datasetname = getattr(data, "name", "Data")
+        slot = self._inputs[index]
+        view = slot.view
+        # reset the (header) view state.
+        view.setModel(None)
+        view.horizontalHeader().setSortIndicator(-1, Qt.AscendingOrder)
+        assert self.tabs.indexOf(view) != -1
+        self.tabs.setTabText(self.tabs.indexOf(view), datasetname)
+        view.dataset = data
+        slot = TableSlot(index, data, table_summary(data), view)
+        view.input_slot = slot
+        self._inputs[index] = slot
+        self._setup_table_view(view, data)
+        self.tabs.setCurrentWidget(view)
 
-                if self.select_rows:
-                    view.setSelectionBehavior(QTableView.SelectRows)
+    @Inputs.data.insert
+    def insert_dataset(self, index: int, data: Table):
+        datasetname = getattr(data, "name", "Data")
+        view = self._create_table_view()
+        slot = TableSlot(None, data, table_summary(data), view)
+        view.dataset = data
+        view.input_slot = slot
+        self._inputs.insert(index, slot)
+        self.tabs.insertTab(index, view, datasetname)
+        self._setup_table_view(view, data)
+        self.tabs.setCurrentWidget(view)
 
-                header = view.horizontalHeader()
-                header.setSectionsMovable(True)
-                header.setSectionsClickable(True)
-                header.setSortIndicatorShown(True)
-                header.setSortIndicator(-1, Qt.AscendingOrder)
+    @Inputs.data.remove
+    def remove_dataset(self, index):
+        slot = self._inputs.pop(index)
+        view = slot.view
+        self.tabs.removeTab(self.tabs.indexOf(view))
+        view.setModel(None)
+        view.hide()
+        view.deleteLater()
 
-                # QHeaderView does not 'reset' the model sort column,
-                # because there is no guaranty (requirement) that the
-                # models understand the -1 sort column.
-                def sort_reset(index, order):
-                    if view.model() is not None and index == -1:
-                        view.model().sort(index, order)
-
-                header.sortIndicatorChanged.connect(sort_reset)
-                self.tabs.addTab(view, datasetname)
-
-            view.dataset = data
-            self.tabs.setCurrentWidget(view)
-
-            self._setup_table_view(view, data)
-            slot = TableSlot(tid, data, table_summary(data), view)
-            view.input_slot = slot
-            self._inputs[tid] = slot
-
-            self.tabs.setCurrentIndex(self.tabs.indexOf(view))
-
-            self._set_input_summary(slot)
-
-            if isinstance(slot.summary.len, concurrent.futures.Future):
-                def update(_):
-                    QMetaObject.invokeMethod(
-                        self, "_update_info", Qt.QueuedConnection)
-
-                slot.summary.len.add_done_callback(update)
-
-        elif tid in self._inputs:
-            slot = self._inputs.pop(tid)
-            view = slot.view
-            view.hide()
-            view.deleteLater()
-            self.tabs.removeTab(self.tabs.indexOf(view))
-
-            current = self.tabs.currentWidget()
-            if current is not None:
-                self._set_input_summary(current.input_slot)
-        else:
-            self._set_input_summary(None)
+        current = self.tabs.currentWidget()
+        if current is not None:
+            self._set_input_summary(current.input_slot)
 
+    def handleNewSignals(self):
+        super().handleNewSignals()
         self.tabs.tabBar().setVisible(self.tabs.count() > 1)
+        data: Optional[Table] = None
+        current = self.tabs.currentWidget()
+        slot = None
+        if current is not None:
+            data = current.dataset
+            slot = current.input_slot
 
-        if data and self.__pending_selected_rows is not None:
+        if slot and isinstance(slot.summary.len, concurrent.futures.Future):
+            def update(_):
+                QMetaObject.invokeMethod(
+                    self, "_update_info", Qt.QueuedConnection)
+            slot.summary.len.add_done_callback(update)
+        self._set_input_summary(slot)
+
+        if data is not None and self.__pending_selected_rows is not None:
             self.selected_rows = self.__pending_selected_rows
             self.__pending_selected_rows = None
         else:
@@ -335,17 +356,12 @@ class OWDataTable(OWWidget):
             self.selected_cols = []
 
         self.set_selection()
-        self.unconditional_commit()
+        self.commit.now()
 
     def _setup_table_view(self, view, data):
         """Setup the `view` (QTableView) with `data` (Orange.data.Table)
         """
-        if data is None:
-            view.setModel(None)
-            return
-
         datamodel = RichTableModel(data)
-
         rowcount = data.approx_len()
 
         if self.color_by_class and data.domain.has_discrete_class:
@@ -355,11 +371,11 @@ class OWDataTable(OWWidget):
             color_schema = None
         if self.show_distributions:
             view.setItemDelegate(
-                gui.TableBarItem(
-                    self, color=self.dist_color, color_schema=color_schema)
+                TableBarItemDelegate(
+                    view, color=self.dist_color, color_schema=color_schema)
             )
         else:
-            view.setItemDelegate(QStyledItemDelegate(self))
+            view.setItemDelegate(TableDataDelegate(view))
 
         # Enable/disable view sorting based on data's type
         view.setSortingEnabled(is_sortable(data))
@@ -403,7 +419,7 @@ class OWDataTable(OWWidget):
         view.setSelectionModel(selmodel)
         view.selectionFinished.connect(self.update_selection)
 
-    #noinspection PyBroadException
+    # noinspection PyBroadException
     def set_corner_text(self, table, text):
         """Set table corner text."""
         # As this is an ugly hack, do everything in
@@ -460,7 +476,7 @@ class OWDataTable(OWWidget):
                 s = btn.style().sizeFromContents(
                     QStyle.CT_HeaderSection,
                     opt, QSize(),
-                    btn).expandedTo(QApplication.globalStrut())
+                    btn)
                 if s.isValid():
                     table.verticalHeader().setMinimumWidth(s.width())
             except Exception:
@@ -490,7 +506,7 @@ class OWDataTable(OWWidget):
                 if not part.nans:
                     return ""
                 perc = 100 * part.nans / (part.nans + part.non_nans)
-                return f" ({perc:.1f} % missing data)"
+                return f" ({perc:.1f} % 缺失值)"
 
             if isinstance(part, SparseArray):
                 tag = "sparse"
@@ -503,54 +519,47 @@ class OWDataTable(OWWidget):
 
         def desc(n, part):
             if n == 0:
-                return f"No {part}s"
-            elif n == 1:
-                return f"1 {part}"
+                return f"无 {part}"
             else:
-                return f"{n} {part}s"
+                return f"{n} {part}"
 
         if slot is None:
-            return ["No data."]
+            return ["无数据."]
         summary = slot.summary
         text = []
         if isinstance(summary, ApproxSummary):
             if summary.len.done():
-                text.append(f"{summary.len.result()} instances")
+                text.append(f"{summary.len.result()} 条数据")
             else:
-                text.append(f"~{summary.approx_len} instances")
+                text.append(f"~{summary.approx_len} 条数据")
         elif isinstance(summary, Summary):
-            text.append(f"{summary.len} instances")
+            text.append(f"{summary.len} 条数据")
             if sum(p.nans for p in [summary.X, summary.Y, summary.M]) == 0:
-                text[-1] += " (no missing data)"
+                text[-1] += " (无缺失值)"
 
-        text.append(desc(len(summary.domain.attributes), "feature")
+        text.append(desc(len(summary.domain.attributes), "个特征")
                     + format_part(summary.X))
 
         if not summary.domain.class_vars:
-            text.append("No target variable.")
+            text.append("无目标.")
         else:
             if len(summary.domain.class_vars) > 1:
                 c_text = desc(len(summary.domain.class_vars), "outcome")
             elif summary.domain.has_continuous_class:
-                c_text = "Numeric outcome"
+                c_text = "目标为数值数据"
             else:
-                c_text = "Target with " \
-                    + desc(len(summary.domain.class_var.values), "value")
+                c_text = "目标有 " \
+                    + desc(len(summary.domain.class_var.values), "个类别")
             text.append(c_text + format_part(summary.Y))
 
-        text.append(desc(len(summary.domain.metas), "meta attribute")
+        text.append(desc(len(summary.domain.metas), "元数据特征")
                     + format_part(summary.M))
         return text
 
-    def _set_output_summary(self, output):
-        summary = len(output) if output else self.info.NoOutput
-        details = format_summary_details(output) if output else ""
-        self.info.set_output_summary(summary, details)
-
     def _on_select_all(self, _):
         data_info = self.tabs.currentWidget().input_slot.summary
         if len(self.selected_rows) == data_info.len \
-                and len(self.selected_cols) == len(data_info.domain):
+                and len(self.selected_cols) == len(data_info.domain.variables):
             self.tabs.currentWidget().clearSelection()
         else:
             self.tabs.currentWidget().selectAll()
@@ -587,7 +596,7 @@ class OWDataTable(OWWidget):
 
     def _on_show_variable_labels_changed(self):
         """The variable labels (var.attribues) visibility was changed."""
-        for slot in self._inputs.values():
+        for slot in self._inputs:
             self._update_variable_labels(slot.view)
 
     def _on_distribution_color_changed(self):
@@ -603,17 +612,17 @@ class OWDataTable(OWWidget):
             else:
                 color_schema = None
             if self.show_distributions:
-                delegate = gui.TableBarItem(self, color=self.dist_color,
-                                            color_schema=color_schema)
+                delegate = TableBarItemDelegate(widget, color=self.dist_color,
+                                                color_schema=color_schema)
             else:
-                delegate = QStyledItemDelegate(self)
+                delegate = TableDataDelegate(widget)
             widget.setItemDelegate(delegate)
         tab = self.tabs.currentWidget()
         if tab:
             tab.reset()
 
     def _on_select_rows_changed(self):
-        for slot in self._inputs.values():
+        for slot in self._inputs:
             selection_model = slot.view.selectionModel()
             selection_model.setSelectBlocks(not self.select_rows)
             if self.select_rows:
@@ -639,7 +648,7 @@ class OWDataTable(OWWidget):
             self._set_input_summary(current.input_slot)
 
     def update_selection(self, *_):
-        self.commit()
+        self.commit.deferred()
 
     def set_selection(self):
         if self.selected_rows and self.selected_cols:
@@ -682,8 +691,10 @@ class OWDataTable(OWWidget):
         assert isinstance(model, TableModel)
 
         row_spans, col_spans = selection_blocks(selection)
-        rows = list(itertools.chain.from_iterable(itertools.starmap(range, row_spans)))
-        cols = list(itertools.chain.from_iterable(itertools.starmap(range, col_spans)))
+        rows = list(itertools.chain.from_iterable(
+            itertools.starmap(range, row_spans)))
+        cols = list(itertools.chain.from_iterable(
+            itertools.starmap(range, col_spans)))
         rows = numpy.array(rows, dtype=numpy.intp)
         # map the rows through the applied sorting (if any)
         rows = model.mapToSourceRows(rows)
@@ -697,6 +708,7 @@ class OWDataTable(OWWidget):
             model = model.sourceModel()
         return model
 
+    @gui.deferred
     def commit(self):
         """
         Commit/send the current selected row/column selection.
@@ -710,7 +722,6 @@ class OWDataTable(OWWidget):
             # Selections of individual instances are not implemented
             # for SqlTables
             if isinstance(table, SqlTable):
-                self._set_output_summary(selected_data)
                 self.Outputs.selected_data.send(selected_data)
                 self.Outputs.annotated_data.send(None)
                 return
@@ -720,7 +731,7 @@ class OWDataTable(OWWidget):
 
             domain = table.domain
 
-            if len(colsel) < len(domain) + len(domain.metas):
+            if len(colsel) < len(domain.variables) + len(domain.metas):
                 # only a subset of the columns is selected
                 allvars = domain.class_vars + domain.metas + domain.attributes
                 columns = [(c, model.headerData(c, Qt.Horizontal,
@@ -740,13 +751,12 @@ class OWDataTable(OWWidget):
                 metas = select_vars(TableModel.Meta)
                 domain = Orange.data.Domain(attrs, class_vars, metas)
 
-            # Avoid a copy if none rows are selected.
+            # Send all data by default
             if not rowsel:
-                selected_data = None
+                selected_data = table
             else:
                 selected_data = table.from_table(domain, table, rowsel)
 
-        self._set_output_summary(selected_data)
         self.Outputs.selected_data.send(selected_data)
         self.Outputs.annotated_data.send(create_annotated_table(table, rowsel))
 
@@ -850,6 +860,8 @@ def is_sortable(table):
 
 if __name__ == "__main__":  # pragma: no cover
     WidgetPreview(OWDataTable).run(
-        [(Table("iris"), "iris"),
-         (Table("brown-selected"), "brown-selected"),
-         (Table("housing"), "housing")])
+        insert_dataset=[
+            (0, Table("iris")),
+            (1, Table("brown-selected")),
+            (2, Table("housing"))
+        ])
