@@ -1,20 +1,16 @@
 # Test methods with long descriptive names can omit docstrings
 # pylint: disable=missing-docstring,pointless-statement,blacklisted-name,unsubscriptable-object
-from unittest.mock import Mock
 import numpy as np
 
 from AnyQt.QtCore import Qt, QItemSelection
 from AnyQt.QtTest import QTest
 
-from orangewidget.widget import StateInfo
-
 from Orange.data import Table, Domain, ContinuousVariable, TimeVariable
 from Orange.preprocess import impute
 from Orange.widgets.data.owimpute import OWImpute, AsDefault, Learner, Method
 from Orange.widgets.tests.base import WidgetTest
 from Orange.widgets.tests.utils import simulate
 from Orange.widgets.utils.itemmodels import select_row
-from Orange.widgets.utils.state_summary import format_summary_details
 
 
 class Foo(Learner):
@@ -214,29 +210,3 @@ class TestOWImpute(WidgetTest):
                         widget.value_combo.isEnabledTo(widget))
         self.assertEqual(varbg.checkedId(), Method.Default)
         self.assertEqual(widget.value_combo.currentIndex(), 1)
-
-    def test_summary(self):
-        """Check if the status bar is updated when data is received"""
-        data = Table("heart_disease")
-        input_sum = self.widget.info.set_input_summary = Mock()
-        output_sum = self.widget.info.set_output_summary = Mock()
-
-        self.send_signal(self.widget.Inputs.data, data)
-        input_sum.assert_called_with(len(data), format_summary_details(data))
-        output = self.get_output(self.widget.Outputs.data)
-        output_sum.assert_called_with(len(output),
-                                      format_summary_details(output))
-
-        varbg = self.widget.variable_button_group
-        varbg.button(Method.AsValue).click()
-        output = self.get_output(self.widget.Outputs.data)
-        output_sum.assert_called_with(len(output),
-                                      format_summary_details(output))
-
-        input_sum.reset_mock()
-        output_sum.reset_mock()
-        self.send_signal(self.widget.Inputs.data, None)
-        input_sum.assert_called_once()
-        self.assertIsInstance(input_sum.call_args[0][0], StateInfo.Empty)
-        output_sum.assert_called_once()
-        self.assertIsInstance(output_sum.call_args[0][0], StateInfo.Empty)
