@@ -2,16 +2,16 @@
 import time
 import warnings
 import unittest
-from unittest.mock import patch, Mock
+from unittest.mock import patch
 
 import numpy as np
 from sklearn.exceptions import ConvergenceWarning
 
-from AnyQt.QtCore import Qt, QItemSelection
-from AnyQt.QtWidgets import QCheckBox
+from AnyQt.QtCore import Qt, QItemSelection, QItemSelectionModel
+from AnyQt.QtWidgets import QCheckBox, QApplication
 
-from orangewidget.widget import StateInfo
 from orangewidget.settings import Context, IncompatibleContext
+from orangewidget.tests.base import GuiTest
 
 from Orange.data import Table, Domain, ContinuousVariable, DiscreteVariable
 from Orange.modelling import RandomForestLearner, SGDLearner
@@ -19,10 +19,10 @@ from Orange.preprocess.score import Scorer
 from Orange.classification import LogisticRegressionLearner
 from Orange.regression import LinearRegressionLearner
 from Orange.projection import PCA
-from Orange.widgets.data.owrank import OWRank, ProblemType, CLS_SCORES, REG_SCORES
+from Orange.widgets.data.owrank import OWRank, ProblemType, CLS_SCORES, \
+    REG_SCORES, TableModel
 from Orange.widgets.tests.base import WidgetTest, datasets
 from Orange.widgets.widget import AttributeList
-from Orange.widgets.utils.state_summary import format_summary_details
 
 
 class SlowScorer(Scorer):
@@ -61,16 +61,16 @@ class TestOWRank(WidgetTest):
 
     def test_input_scorer(self):
         """Check widget's scorer with scorer on the input"""
-        self.assertEqual(self.widget.scorers, {})
+        self.assertEqual(self.widget.scorers, [])
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 1)
         self.wait_until_finished()
-        value = self.widget.scorers[1]
+        value = self.widget.scorers[0]
         self.assertEqual(self.log_reg, value.scorer)
         self.assertIsInstance(value.scorer, Scorer)
 
     def test_input_scorer_fitter(self):
         heart_disease = Table('heart_disease')
-        self.assertEqual(self.widget.scorers, {})
+        self.assertEqual(self.widget.scorers, [])
 
         model = self.widget.ranksModel
 
@@ -96,14 +96,14 @@ class TestOWRank(WidgetTest):
                         self.assertIn(name, last_column)
 
                 self.send_signal("评分器(Scorer)", None, 1)
-                self.assertEqual(self.widget.scorers, {})
+                self.assertEqual(self.widget.scorers, [])
 
     def test_input_scorer_disconnect(self):
         """Check widget's scorer after disconnecting scorer on the input"""
         self.send_signal(self.widget.Inputs.scorer, self.log_reg, 1)
         self.assertEqual(len(self.widget.scorers), 1)
         self.send_signal(self.widget.Inputs.scorer, None, 1)
-        self.assertEqual(self.widget.scorers, {})
+        self.assertEqual(self.widget.scorers, [])
 
     def test_output_data(self):
         """Check data on the output after apply"""
@@ -310,23 +310,24 @@ class TestOWRank(WidgetTest):
         order1 = self.widget.ranksModel.mapToSourceRows(...).tolist()
         self._get_checkbox('FCBF').setChecked(True)
         self.wait_until_finished()
-        self.widget.ranksView.horizontalHeader().setSortIndicator(3, Qt.DescendingOrder)
+        self.widget.ranksView.horizontalHeader().setSortIndicator(4, Qt.DescendingOrder)
         order2 = self.widget.ranksModel.mapToSourceRows(...).tolist()
         self.assertNotEqual(order1, order2)
 
     def test_scores_nan_sorting(self):
         """Check NaNs are sorted last"""
         data = self.iris.copy()
-        data.get_column_view('petal length')[0][:] = np.nan
+        with data.unlocked():
+            data.get_column_view('petal length')[0][:] = np.nan
         self.send_signal(self.widget.Inputs.data, data)
         self.wait_until_finished()
 
         # Assert last row is all nan
         for order in (Qt.AscendingOrder,
                       Qt.DescendingOrder):
-            self.widget.ranksView.horizontalHeader().setSortIndicator(1, order)
+            self.widget.ranksView.horizontalHeader().setSortIndicator(2, order)
             last_row = self.widget.ranksModel[self.widget.ranksModel.mapToSourceRows(...)[-1]]
-            np.testing.assert_array_equal(last_row, np.repeat(np.nan, 3))
+            np.testing.assert_array_equal(last_row[1:], np.repeat(np.nan, 3))
 
     def test_default_sort_indicator(self):
         self.send_signal(self.widget.Inputs.data, self.iris)
@@ -397,7 +398,7 @@ class TestOWRank(WidgetTest):
 
         # Sort by number of values and set selection to attributes with most
         # values. This must select the top 4 rows.
-        self.widget.ranksView.horizontalHeader().setSortIndicator(0, Qt.DescendingOrder)
+        self.widget.ranksView.horizontalHeader().setSortIndicator(1, Qt.DescendingOrder)
         w.selectionMethod = w.SelectManual
         w.selected_attrs = [dom["chest pain"], dom["rest ECG"],
                             dom["slope peak exc ST"], dom["thal"]]
@@ -406,6 +407,69 @@ class TestOWRank(WidgetTest):
             sorted({idx.row() for idx in w.ranksView.selectedIndexes()}),
             [0, 1, 2, 3])
 
+    def test_resorting_and_selection(self):
+        def sortby(col):
+            model.sort(col)
+            view.horizontalHeader().sectionClicked.emit(col)
+            QApplication.processEvents()
+
+        Names, Values, Gain = range(3)
+
+        w = self.widget
+
+        data = Table("heart_disease")
+        self.send_signal(w.Inputs.data, data)
+        self.wait_until_finished()
+
+        first4 = set(sorted((var.name for var in data.domain.attributes), key=str.lower)[:4])
+
+        view = w.ranksView
+        model = w.ranksModel
+        selModel = view.selectionModel()
+        columnCount = model.columnCount()
+
+        w.selectionMethod = w.SelectNBest
+        w.nSelected = 4
+
+        # Sort by gain ratio, store selection
+        sortby(Gain)
+        gain_sel_4 = w.selected_attrs[:]
+        self.assertEqual(len(gain_sel_4), 4)
+
+        # Sort by names or number of values: selection unchanged
+        sortby(Values)
+        self.assertEqual(w.selected_attrs[:], gain_sel_4)
+
+        sortby(Names)
+        self.assertEqual(w.selected_attrs[:], gain_sel_4)
+
+        # Select first four (alphabetically)
+        w.selectionMethod = w.SelectManual
+        selection = QItemSelection(
+            model.index(0, 0),
+            model.index(3, columnCount - 1)
+        )
+        selModel.select(selection, QItemSelectionModel.ClearAndSelect)
+        # Sanity check
+        self.assertEqual({var.name for var in w.selected_attrs}, first4)
+
+        # Manual sorting: sorting by score does not change selection
+        sortby(Gain)
+        self.assertEqual({var.name for var in w.selected_attrs}, first4)
+
+        # Sort by first four, again
+        sortby(Names)
+        # Sanity check
+        self.assertEqual({var.name for var in w.selected_attrs}, first4)
+
+        w.selectionMethod = w.SelectNBest
+        # Sanity check
+        self.assertEqual({var.name for var in w.selected_attrs}, first4)
+
+        # Sorting by gain must change selection
+        sortby(Gain)
+        self.assertEqual(set(w.selected_attrs), set(gain_sel_4))
+
     def test_auto_send(self):
         widget = self.widget
         model = widget.ranksModel
@@ -472,28 +536,6 @@ class TestOWRank(WidgetTest):
 
         self.assertEqual(len(output), len(self.iris))
 
-    def test_summary(self):
-        """Check if the status bar is updated when data is received"""
-        data = self.iris
-        input_sum = self.widget.info.set_input_summary = Mock()
-        output_sum = self.widget.info.set_output_summary = Mock()
-
-        self.send_signal(self.widget.Inputs.data, data)
-        self.wait_until_finished()
-        input_sum.assert_called_with(len(data), format_summary_details(data))
-        output = self.get_output(self.widget.Outputs.reduced_data)
-        output_sum.assert_called_with(len(output),
-                                      format_summary_details(output))
-
-        input_sum.reset_mock()
-        output_sum.reset_mock()
-        self.send_signal(self.widget.Inputs.data, None)
-        self.wait_until_finished()
-        input_sum.assert_called_once()
-        self.assertIsInstance(input_sum.call_args[0][0], StateInfo.Empty)
-        output_sum.assert_called_once()
-        self.assertIsInstance(output_sum.call_args[0][0], StateInfo.Empty)
-
     def test_concurrent_cancel(self):
         """
         Send one signal after another. It test if the first process get
@@ -508,5 +550,24 @@ class TestOWRank(WidgetTest):
         self.assertEqual(len(output), len(self.housing))
 
 
+class TestRankModel(GuiTest):
+    @staticmethod
+    def test_argsort():
+        func = TableModel()._argsortData  # pylint: disable=protected-access
+        assert_equal = np.testing.assert_equal
+
+        test_array = np.array([4.2, 7.2, np.nan, 1.3, np.nan])
+        assert_equal(func(test_array, Qt.AscendingOrder)[:3], [3, 0, 1])
+        assert_equal(func(test_array, Qt.DescendingOrder)[:3], [1, 0, 3])
+
+        test_array = np.array([4, 7, 2])
+        assert_equal(func(test_array, Qt.AscendingOrder), [2, 0, 1])
+        assert_equal(func(test_array, Qt.DescendingOrder), [1, 0, 2])
+
+        test_array = np.array(["Bertha", "daniela", "ann", "Cecilia"])
+        assert_equal(func(test_array, Qt.AscendingOrder), [2, 0, 3, 1])
+        assert_equal(func(test_array, Qt.DescendingOrder), [1, 3, 0, 2])
+
+
 if __name__ == "__main__":
     unittest.main()
