@@ -9,9 +9,11 @@ import copy
 from functools import partial, reduce
 
 from concurrent.futures import Future
-from collections import OrderedDict, namedtuple
+from collections import OrderedDict
 from itertools import count
-from typing import Any, Optional, List, Dict, Callable
+from typing import (
+    Any, Optional, List, Dict, Callable, Sequence, NamedTuple, Tuple
+)
 
 import numpy as np
 import baycomp
@@ -20,7 +22,8 @@ from AnyQt import QtGui
 from AnyQt.QtCore import Qt, QSize, QThread
 from AnyQt.QtCore import pyqtSlot as Slot
 from AnyQt.QtGui import QStandardItem, QDoubleValidator
-from AnyQt.QtWidgets import QHeaderView, QTableWidget, QLabel
+from AnyQt.QtWidgets import \
+    QHeaderView, QTableWidget, QLabel, QComboBox, QSizePolicy
 
 from Orange.base import Learner
 import Orange.classification
@@ -39,18 +42,16 @@ from Orange.widgets.evaluate.utils import \
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.utils.concurrent import ThreadExecutor, TaskState
-from Orange.widgets.utils.state_summary import (format_multiple_summaries,
-                                                format_summary_details)
-from Orange.widgets.widget import OWWidget, Msg, Input, Output
+from Orange.widgets.widget import OWWidget, Msg, Input, MultiInput, Output
 
 log = logging.getLogger(__name__)
 
-InputLearner = namedtuple(
-    "InputLearner",
-    ["learner",  # :: Orange.base.Learner
-     "results",  # :: Option[Try[Orange.evaluation.Results]]
-     "stats"]    # :: Option[Sequence[Try[float]]]
-)
+
+class InputLearner(NamedTuple):
+    learner:  Orange.base.Learner
+    results: Optional['Try[Orange.evaluation.Results]']
+    stats: Optional[Sequence['Try[float]']]
+    key: Any
 
 
 class Try(abc.ABC):
@@ -136,23 +137,27 @@ class OWTestAndScore(OWWidget):
     description = "交叉验证精度估计。"
     icon = "icons/TestLearners1.svg"
     priority = 100
-    keywords = ['Cross Validation', 'CV', 'xunlianheceshi',
-                'xunlianyuceshi', 'ceshi', 'moxing']
+    keywords = ['Cross Validation', 'CV', 'ceshi']
     replaces = ["Orange.widgets.evaluate.owtestlearners.OWTestLearners"]
-    category = 'evaluate'
-
+    category = '评估(Evaluate)'
 
     class Inputs:
         train_data = Input("数据(Data)", Table, default=True, replaces=['Data'])
         test_data = Input("测试数据(Test Data)", Table, replaces=['Test Data'])
-        learner = Input("学习器(Learner)", Learner, multiple=True, replaces=['Learner'])
-        preprocessor = Input("预处理器(Preprocessor)", Preprocess, replaces=['Preprocessor'])
+        learner = MultiInput(
+            "学习器(Learner)", Learner, filter_none=True, replaces=['Learner']
+        )
+        preprocessor = Input("预处理器(Preprocessor)",
+                             Preprocess, replaces=['Preprocessor'])
 
     class Outputs:
-        predictions = Output("预测(Predictions)", Table, replaces=['Predictions'])
-        evaluations_results = Output("评价结果(Evaluation Results)", Results, replaces=['Evaluation Results'])
+        predictions = Output("预测(Predictions)", Table,
+                             replaces=['Predictions'])
+        evaluations_results = Output(
+            "评价结果(Evaluation Results)", Results, replaces=['Evaluation Results'])
 
     settings_version = 3
+    buttons_area_orientation = None
     UserAdviceMessages = [
         widget.Message(
             "Click on the table header to select shown columns",
@@ -213,12 +218,16 @@ class OWTestAndScore(OWWidget):
         scores_not_computed = Msg("Some scores could not be computed.")
         test_data_unused = Msg("Test data is present but unused. "
                                "Select 'Test on test data' to use it.")
+        cant_stratify = \
+            Msg("Can't run stratified {}-fold cross validation; "
+                "the least common class has only {} instances.")
 
     class Information(OWWidget.Information):
         data_sampled = Msg("Train data has been sampled")
         test_data_sampled = Msg("Test data has been sampled")
         test_data_transformed = Msg(
             "Test data has been transformed to match the train data.")
+        cant_stratify_numeric = Msg("Stratification is ignored for regression")
 
     def __init__(self):
         super().__init__()
@@ -230,8 +239,10 @@ class OWTestAndScore(OWWidget):
         self.test_data_missing_vals = False
         self.scorers = []
         self.__pending_comparison_criterion = self.comparison_criterion
-
-        #: An Ordered dictionary with current inputs and their testing results.
+        self.__id_gen = count()
+        self._learner_inputs = []  # type: List[Tuple[Any, Learner]]
+        #: An Ordered dictionary with current inputs and their testing results
+        #: (keyed by ids generated by __id_gen).
         self.learners = OrderedDict()  # type: Dict[Any, Input]
 
         self.__state = State.Waiting
@@ -241,10 +252,7 @@ class OWTestAndScore(OWWidget):
         self.__task = None  # type: Optional[TaskState]
         self.__executor = ThreadExecutor()
 
-        self.info.set_input_summary(self.info.NoInput)
-        self.info.set_output_summary(self.info.NoOutput)
-
-        sbox = gui.vBox(self.controlArea, "抽样")
+        sbox = gui.vBox(self.controlArea, box=True)
         rbox = gui.radioButtons(
             sbox, self, "resampling", callback=self._param_changed)
 
@@ -287,37 +295,41 @@ class OWTestAndScore(OWWidget):
         gui.appendRadioButton(rbox, "测试训练数据")
         gui.appendRadioButton(rbox, "测试测试数据")
 
-        self.cbox = gui.vBox(self.controlArea, "目标类别")
-        self.class_selection_combo = gui.comboBox(
-            self.cbox, self, "class_selection", items=[],
-            sendSelectedValue=True, contentsLength=8, searchable=True,
-            callback=self._on_target_class_changed
-        )
-
-        self.modcompbox = box = gui.vBox(self.controlArea, "模型比较")
-        gui.comboBox(
-            box, self, "comparison_criterion",
-            callback=self.update_comparison_table)
-
-        hbox = gui.hBox(box)
-        gui.checkBox(hbox, self, "use_rope",
-                     "可忽略区别: ",
-                     callback=self._on_use_rope_changed)
-        gui.lineEdit(hbox, self, "rope", validator=QDoubleValidator(),
-                     controlWidth=70, callback=self.update_comparison_table,
-                     alignment=Qt.AlignRight)
-        self.controls.rope.setEnabled(self.use_rope)
-
         gui.rubber(self.controlArea)
+
         self.score_table = ScoreTable(self)
         self.score_table.shownScoresChanged.connect(self.update_stats_model)
         view = self.score_table.view
         view.setSizeAdjustPolicy(view.AdjustToContents)
 
-        box = gui.vBox(self.mainArea, "评价结果")
-        box.layout().addWidget(self.score_table.view)
+        self.results_box = gui.vBox(self.mainArea, box=True)
+        self.cbox = gui.hBox(self.results_box)
+        self.class_selection_combo = gui.comboBox(
+            self.cbox, self, "class_selection", items=[],
+            label="评估目标", orientation=Qt.Horizontal,
+            sendSelectedValue=True, searchable=True, contentsLength=25,
+            callback=self._on_target_class_changed
+        )
+        self.cbox.layout().addStretch(100)
+        self.class_selection_combo.setMaximumContentsLength(30)
+        self.results_box.layout().addWidget(self.score_table.view)
+
+        gui.separator(self.mainArea, 16)
+        self.compbox = box = gui.vBox(self.mainArea, box=True)
+        cbox = gui.comboBox(
+            box, self, "comparison_criterion", label="模型比较依据:",
+            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
+            sizePolicy=(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed),
+            orientation=Qt.Horizontal, callback=self.update_comparison_table).box
+
+        gui.separator(cbox, 8)
+        gui.checkBox(cbox, self, "use_rope", "可忽略区别: ",
+                     callback=self._on_use_rope_changed)
+        gui.lineEdit(cbox, self, "rope", validator=QDoubleValidator(),
+                     controlWidth=50, callback=self.update_comparison_table,
+                     alignment=Qt.AlignRight)
+        self.controls.rope.setEnabled(self.use_rope)
 
-        self.compbox = box = gui.vBox(self.mainArea, box='模型比较')
         table = self.comparison_table = QTableWidget(
             wordWrap=False, editTriggers=QTableWidget.NoEditTriggers,
             selectionMode=QTableWidget.NoSelection)
@@ -342,9 +354,9 @@ class OWTestAndScore(OWWidget):
             "\"列模型\"的评价指标. "
             "值小说明概率很小, 区别可以忽略不计.</small>", wordWrap=True))
 
-    @staticmethod
-    def sizeHint():
-        return QSize(780, 1)
+    def sizeHint(self):
+        sh = super().sizeHint()
+        return QSize(780, sh.height())
 
     def _update_controls(self):
         self.fold_feature = None
@@ -361,22 +373,35 @@ class OWTestAndScore(OWWidget):
             self.resampling = OWTestAndScore.KFold
 
     @Inputs.learner
-    def set_learner(self, learner, key):
+    def set_learner(self, index: int, learner: Learner):
         """
-        Set the input `learner` for `key`.
+        Set the input `learner` at `index`.
 
         Parameters
         ----------
-        learner : Optional[Orange.base.Learner]
-        key : Any
+        index: int
+        learner: Orange.base.Learner
         """
-        if key in self.learners and learner is None:
-            # Removed
-            self._invalidate([key])
-            del self.learners[key]
-        elif learner is not None:
-            self.learners[key] = InputLearner(learner, None, None)
-            self._invalidate([key])
+        key, _ = self._learner_inputs[index]
+        slot = self.learners[key]
+        self.learners[key] = slot._replace(learner=learner, results=None)
+        self._invalidate([key])
+
+    @Inputs.learner.insert
+    def insert_learner(self, index: int, learner: Learner):
+        key = next(self.__id_gen)
+        self._learner_inputs.insert(index, (key, learner))
+        self.learners[key] = InputLearner(learner, None, None, key)
+        self.learners = {key: self.learners[key]
+                         for key, _ in self._learner_inputs}
+        self._invalidate([key])
+
+    @Inputs.learner.remove
+    def remove_learner(self, index: int):
+        key, _ = self._learner_inputs[index]
+        self._invalidate([key])
+        self._learner_inputs.pop(index)
+        self.learners.pop(key)
 
     @Inputs.train_data
     def set_train_data(self, data):
@@ -517,15 +542,6 @@ class OWTestAndScore(OWWidget):
             if self.__pending_comparison_criterion < len(self.scorers):
                 self.comparison_criterion = self.__pending_comparison_criterion
             self.__pending_comparison_criterion = None
-        self._update_compbox_title()
-
-    def _update_compbox_title(self):
-        criterion = self.comparison_criterion
-        if criterion < len(self.scorers):
-            scorer = self.scorers[criterion]()
-            self.compbox.setTitle(f"根据 {scorer.name} 比较模型")
-        else:
-            self.compbox.setTitle(f"模型比较")
 
     @Inputs.preprocessor
     def set_preprocessor(self, preproc):
@@ -541,27 +557,9 @@ class OWTestAndScore(OWWidget):
         self.score_table.update_header(self.scorers)
         self._update_view_enabled()
         self.update_stats_model()
-        self.set_input_summary()
         if self.__needupdate:
             self.__update()
 
-    def set_input_summary(self):
-        summary, details, kwargs = self.info.NoInput, "", {}
-        if self.data and self.test_data:
-            summary = f"{self.info.format_number(len(self.data))}," \
-                      f" {self.info.format_number(len(self.test_data))}"
-            details = format_multiple_summaries([
-                ("Data", self.data), ("Test data", self.test_data)
-            ])
-            kwargs = {"format": Qt.RichText}
-        elif self.data and not self.test_data:
-            summary, details = len(
-                self.data), format_summary_details(self.data)
-        elif self.test_data and not self.data:
-            summary = len(self.test_data)
-            details = format_summary_details(self.test_data)
-        self.info.set_input_summary(summary, details, **kwargs)
-
     def kfold_changed(self):
         self.resampling = OWTestAndScore.KFold
         self._param_changed()
@@ -575,13 +573,12 @@ class OWTestAndScore(OWWidget):
         self._param_changed()
 
     def _param_changed(self):
-        self.modcompbox.setEnabled(self.resampling == OWTestAndScore.KFold)
         self._update_view_enabled()
         self._invalidate()
         self.__update()
 
     def _update_view_enabled(self):
-        self.comparison_table.setEnabled(
+        self.compbox.setEnabled(
             self.resampling == OWTestAndScore.KFold
             and len(self.learners) > 1
             and self.data is not None)
@@ -725,7 +722,6 @@ class OWTestAndScore(OWWidget):
 
     def _scores_by_folds(self, slots):
         scorer = self.scorers[self.comparison_criterion]()
-        self._update_compbox_title()
         if scorer.is_binary:
             if self.class_selection != self.TARGET_AVERAGE:
                 class_var = self.data.domain.class_var
@@ -869,10 +865,6 @@ class OWTestAndScore(OWWidget):
             except MemoryError:
                 self.Error.memory_error()
 
-        summary = len(predictions) if predictions else self.info.NoOutput
-        details = format_summary_details(predictions) if predictions else ""
-        self.info.set_output_summary(summary, details)
-
         self.Outputs.evaluations_results.send(combined)
         self.Outputs.predictions.send(predictions)
 
@@ -929,6 +921,8 @@ class OWTestAndScore(OWWidget):
         self.Warning.test_data_unused.clear()
         self.Error.test_data_incompatible.clear()
         self.Warning.test_data_missing.clear()
+        self.Warning.cant_stratify.clear()
+        self.Information.cant_stratify_numeric.clear()
         self.Information.test_data_transformed(
             shown=self.resampling == self.TestOnTest
             and self.data is not None
@@ -939,7 +933,7 @@ class OWTestAndScore(OWWidget):
         self.Error.too_many_folds.clear()
         self.error()
 
-        # check preconditions and return early
+        # check preconditions and return early or show warnings
         if self.data is None:
             self.__state = State.Waiting
             self.commit()
@@ -948,12 +942,24 @@ class OWTestAndScore(OWWidget):
             self.__state = State.Waiting
             self.commit()
             return
-        if self.resampling == OWTestAndScore.KFold and \
-                len(self.data) < self.NFolds[self.n_folds]:
-            self.Error.too_many_folds()
-            self.__state = State.Waiting
-            self.commit()
-            return
+        if self.resampling == OWTestAndScore.KFold:
+            k = self.NFolds[self.n_folds]
+            if len(self.data) < k:
+                self.Error.too_many_folds()
+                self.__state = State.Waiting
+                self.commit()
+                return
+            do_stratify = self.cv_stratified
+            if do_stratify:
+                if self.data.domain.class_var.is_discrete:
+                    least = min(filter(None,
+                                       np.bincount(self.data.Y.astype(int))))
+                    if least < k:
+                        self.Warning.cant_stratify(k, least)
+                        do_stratify = False
+                else:
+                    self.Information.cant_stratify_numeric()
+                    do_stratify = False
 
         elif self.resampling == OWTestAndScore.TestOnTest:
             if self.test_data is None:
@@ -993,7 +999,8 @@ class OWTestAndScore(OWWidget):
             if self.resampling == OWTestAndScore.KFold:
                 sampler = Orange.evaluation.CrossValidation(
                     k=self.NFolds[self.n_folds],
-                    random_state=rstate)
+                    random_state=rstate,
+                    stratified=do_stratify)
             elif self.resampling == OWTestAndScore.FeatureFold:
                 sampler = Orange.evaluation.CrossValidationFeature(
                     feature=self.fold_feature)
@@ -1227,5 +1234,5 @@ if __name__ == "__main__":  # pragma: no cover
     WidgetPreview(OWTestAndScore).run(
         set_train_data=preview_data,
         set_test_data=preview_data,
-        set_learner=[(learner, i) for i, learner in enumerate(prev_learners)]
+        insert_learner=list(enumerate(prev_learners))
     )
