@@ -10,7 +10,6 @@ from Orange.data import Table, Domain, ContinuousVariable, DiscreteVariable
 from Orange.widgets.tests.base import WidgetTest
 from Orange.widgets.tests.utils import simulate, possible_duplicate_table
 from Orange.widgets.unsupervised.owmanifoldlearning import OWManifoldLearning
-from Orange.widgets.utils.state_summary import format_summary_details
 
 
 class TestOWManifoldLearning(WidgetTest):
@@ -102,7 +101,7 @@ class TestOWManifoldLearning(WidgetTest):
             self.assertFalse(self.widget.Error.manifold_error.is_shown())
 
         simulate.combobox_run_through_all(
-            self.widget.tsne_editor.metric_combo, callback=__callback,
+            self.widget.tsne_editor.controls.metric_index, callback=__callback,
         )
 
     def test_unique_domain(self):
@@ -154,29 +153,8 @@ class TestOWManifoldLearning(WidgetTest):
             self.assertTrue(self.widget.Error.out_of_memory.is_shown())
 
     def test_unconditional_commit_on_new_signal(self):
-        with patch.object(self.widget, 'unconditional_apply') as apply:
+        with patch.object(self.widget.commit, 'now') as apply:
             self.widget.auto_apply = False
             apply.reset_mock()
             self.send_signal(self.widget.Inputs.data, self.iris)
             apply.assert_called()
-
-    def test_summary(self):
-        """Check if the status bar updates when data on input"""
-        info = self.widget.info
-        no_input, no_output = "No data on input", "No data on output"
-
-        self.send_signal(self.widget.Inputs.data, self.iris)
-        summary, details = f"{len(self.iris)}", format_summary_details(
-            self.iris)
-        self.assertEqual(info._StateInfo__input_summary.brief, summary)
-        self.assertEqual(info._StateInfo__input_summary.details, details)
-        output = self.get_output(self.widget.Outputs.transformed_data)
-        summary, details = f"{len(output)}", format_summary_details(output)
-        self.assertEqual(info._StateInfo__output_summary.brief, summary)
-        self.assertEqual(info._StateInfo__output_summary.details, details)
-
-        self.send_signal(self.widget.Inputs.data, None)
-        self.assertEqual(info._StateInfo__input_summary.brief, "-")
-        self.assertEqual(info._StateInfo__input_summary.details, no_input)
-        self.assertEqual(info._StateInfo__output_summary.brief, "-")
-        self.assertEqual(info._StateInfo__output_summary.details, no_output)
