@@ -11,10 +11,9 @@ from Orange.data.table import Table
 from Orange.widgets import gui, settings
 from Orange.widgets.utils.annotated_data import (
     create_annotated_table,
-    ANNOTATED_DATA_SIGNAL_Chinese_NAME
+    ANNOTATED_DATA_SIGNAL_NAME,
 )
 from Orange.widgets.utils.signals import Input, Output
-from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.visualize.pythagorastreeviewer import (
     PythagorasTreeViewer,
@@ -27,8 +26,7 @@ from Orange.widgets.visualize.utils.owlegend import (
     OWDiscreteLegend,
     OWContinuousLegend,
 )
-from Orange.widgets.visualize.utils.scene import \
-    UpdateItemsOnSelectGraphicsScene
+from Orange.widgets.visualize.utils.scene import UpdateItemsOnSelectGraphicsScene
 from Orange.widgets.visualize.utils.tree.skltreeadapter import SklTreeAdapter
 from Orange.widgets.visualize.utils.tree.treeadapter import TreeAdapter
 from Orange.widgets.visualize.utils.view import (
@@ -40,23 +38,25 @@ from Orange.widgets.widget import OWWidget
 
 
 class OWPythagorasTree(OWWidget):
-    name = '毕达哥拉斯树(Pythagorean Tree)'
-    description = '类似树结构的毕达哥拉斯树可视化。'
-    icon = 'icons/PythagoreanTree.svg'
-    keywords = ["fractal", 'bidagelasishu', 'gougushu']
-    category = 'visualize'
+    name = "毕达哥拉斯树(Pythagorean Tree)"
+    description = "类似树结构的毕达哥拉斯树可视化。"
+    icon = "icons/PythagoreanTree.svg"
+    keywords = ["fractal", "bidagelasishu", "gougushu"]
+    category = "可视化(Visualize)"
 
     priority = 1000
 
     class Inputs:
-        tree = Input("树(Tree)", TreeModel, replaces=['Tree'])
+        tree = Input("树(Tree)", TreeModel, replaces=["Tree"])
 
     class Outputs:
-        selected_data = Output("选定的数据(Selected Data)", Table, default=True, replaces=['Selected Data'])
-        annotated_data = Output(ANNOTATED_DATA_SIGNAL_Chinese_NAME, Table, replaces=['Data'])
+        selected_data = Output(
+            "选定的数据(Selected Data)", Table, default=True, replaces=["Selected Data"]
+        )
+        annotated_data = Output("数据(Data)", Table, replaces=["Data"])
 
     # Enable the save as feature
-    graph_name = 'scene'
+    graph_name = "scene"
 
     # Settings
     settingsHandler = settings.ClassValuesContextHandler()
@@ -69,8 +69,8 @@ class OWPythagorasTree(OWWidget):
     show_legend = settings.Setting(False)
 
     LEGEND_OPTIONS = {
-        'corner': Anchorable.BOTTOM_RIGHT,
-        'offset': (10, 10),
+        "corner": Anchorable.BOTTOM_RIGHT,
+        "offset": (10, 10),
     }
 
     def __init__(self):
@@ -86,53 +86,83 @@ class OWPythagorasTree(OWWidget):
 
         # Different methods to calculate the size of squares
         self.SIZE_CALCULATION = [
-            ('Normal', lambda x: x, '正常'),
-            ('Square root', lambda x: sqrt(x), '平方根'),
-            ('Logarithmic', lambda x: log(x * self.size_log_scale + 1), '对数'),
+            ("Normal", lambda x: x, "正常"),
+            ("Square root", lambda x: sqrt(x), "平方根"),
+            ("Logarithmic", lambda x: log(x * self.size_log_scale + 1), "对数"),
         ]
 
         # CONTROL AREA
         # Tree info area
-        box_info = gui.widgetBox(self.controlArea, '树信息')
+        box_info = gui.widgetBox(self.controlArea, "树信息")
         self.infolabel = gui.widgetLabel(box_info)
-        self.info.set_output_summary(self.info.NoOutput)
 
         # Display settings area
-        box_display = gui.widgetBox(self.controlArea, '显示设置')
+        box_display = gui.widgetBox(self.controlArea, "显示设置")
+        # maxValue is set to a wide three-digit number to probably ensure the
+        # proper label width. The maximum is later set to match the tree depth
         self.depth_slider = gui.hSlider(
-            box_display, self, 'depth_limit', label='深度', ticks=False,
-            callback=self.update_depth)
+            box_display,
+            self,
+            "depth_limit",
+            label="深度",
+            ticks=False,
+            maxValue=900,
+            callback=self.update_depth,
+        )
         self.target_class_combo = gui.comboBox(
-            box_display, self, 'target_class_index', label='目标类别',
-            orientation=Qt.Horizontal, items=[], contentsLength=8,
-            searchable=True,
-            callback=self.update_colors)
-        self.size_calc_combo = gui.comboBox(
-            box_display, self, 'size_calc_idx', label='大小',
+            box_display,
+            self,
+            "target_class_index",
+            label="目标类别",
             orientation=Qt.Horizontal,
-            items=list(zip(*self.SIZE_CALCULATION))[2], contentsLength=8,
-            callback=self.update_size_calc)
+            items=[],
+            contentsLength=8,
+            searchable=True,
+            callback=self.update_colors,
+        )
+        self.size_calc_combo = gui.comboBox(
+            box_display,
+            self,
+            "size_calc_idx",
+            label="大小",
+            orientation=Qt.Horizontal,
+            items=list(zip(*self.SIZE_CALCULATION))[2],
+            contentsLength=8,
+            callback=self.update_size_calc,
+        )
         self.log_scale_box = gui.hSlider(
-            box_display, self, 'size_log_scale',
-            label='对数比例因子', minValue=1, maxValue=100, ticks=False,
-            callback=self.invalidate_tree)
+            box_display,
+            self,
+            "size_log_scale",
+            label="对数比例因子",
+            minValue=1,
+            maxValue=100,
+            ticks=False,
+            callback=self.invalidate_tree,
+        )
 
         # Plot properties area
-        box_plot = gui.widgetBox(self.controlArea, '绘图属性')
+        box_plot = gui.widgetBox(self.controlArea, "绘图属性")
         self.cb_show_tooltips = gui.checkBox(
-            box_plot, self, 'tooltips_enabled', label='启动工具提示',
-            callback=self.update_tooltip_enabled)
+            box_plot,
+            self,
+            "tooltips_enabled",
+            label="启动工具提示",
+            callback=self.update_tooltip_enabled,
+        )
         self.cb_show_legend = gui.checkBox(
-            box_plot, self, 'show_legend', label='显示图例',
-            callback=self.update_show_legend)
+            box_plot,
+            self,
+            "show_legend",
+            label="显示图例",
+            callback=self.update_show_legend,
+        )
 
-        gui.button(self.controlArea, self, label="重新绘制", callback=self.redraw)
-
-        # Stretch to fit the rest of the unsused area
         gui.rubber(self.controlArea)
 
-        self.controlArea.setSizePolicy(
-            QSizePolicy.Preferred, QSizePolicy.Expanding)
+        gui.button(self.buttonsArea, self, label="重新绘制", callback=self.redraw)
+
+        self.controlArea.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
 
         # MAIN AREA
         self.scene = TreeGraphicsScene(self)
@@ -255,10 +285,11 @@ class OWPythagorasTree(OWWidget):
         self._update_legend_visibility()
 
     def _update_info_box(self):
-        self.infolabel.setText('节点: {}\n深度: {}'.format(
-            self.tree_adapter.num_nodes,
-            self.tree_adapter.max_depth
-        ))
+        self.infolabel.setText(
+            "节点: {}\n深度: {}".format(
+                self.tree_adapter.num_nodes, self.tree_adapter.max_depth
+            )
+        )
 
     def _update_depth_slider(self):
         self.depth_slider.parent().setEnabled(True)
@@ -272,10 +303,11 @@ class OWPythagorasTree(OWWidget):
     def _update_log_scale_slider(self):
         """On calc method combo box changed."""
         self.log_scale_box.parent().setEnabled(
-            self.SIZE_CALCULATION[self.size_calc_idx][0] == 'Logarithmic')
+            self.SIZE_CALCULATION[self.size_calc_idx][0] == "Logarithmic"
+        )
 
     def _clear_info_box(self):
-        self.infolabel.setText('没有树输入')
+        self.infolabel.setText("没有树输入")
 
     def _clear_depth_slider(self):
         self.depth_slider.parent().setEnabled(False)
@@ -311,20 +343,17 @@ class OWPythagorasTree(OWWidget):
     def commit(self):
         """Commit the selected data to output."""
         if self.data is None:
-            self.info.set_output_summary(self.info.NoOutput)
             self.Outputs.selected_data.send(None)
             self.Outputs.annotated_data.send(None)
             return
 
         nodes = [
-            i.tree_node.label for i in self.scene.selectedItems()
+            i.tree_node.label
+            for i in self.scene.selectedItems()
             if isinstance(i, SquareGraphicsItem)
         ]
         data = self.tree_adapter.get_instances_in_nodes(nodes)
 
-        summary = len(data) if data else self.info.NoOutput
-        details = format_summary_details(data) if data else ""
-        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(data)
         selected_indices = self.tree_adapter.get_indices(nodes)
         self.Outputs.annotated_data.send(
@@ -337,15 +366,18 @@ class OWPythagorasTree(OWWidget):
 
     def _update_target_class_combo(self):
         self._clear_target_class_combo()
-        label = [x for x in self.target_class_combo.parent().children()
-                 if isinstance(x, QLabel)][0]
+        label = [
+            x
+            for x in self.target_class_combo.parent().children()
+            if isinstance(x, QLabel)
+        ][0]
 
         if self.data.domain.has_discrete_class:
-            label_text = '目标类别'
+            label_text = "目标类别"
             values = [c.title() for c in self.data.domain.class_vars[0].values]
-            values.insert(0, 'None')
+            values.insert(0, "None")
         else:
-            label_text = 'Node color'
+            label_text = "Node color"
             values = list(ContinuousTreeNode.COLOR_METHODS.keys())
         label.setText(label_text)
         self.target_class_combo.addItems(values)
@@ -363,13 +395,16 @@ class OWPythagorasTree(OWWidget):
 
     def _classification_update_legend_colors(self):
         if self.target_class_index == 0:
-            self.legend = OWDiscreteLegend(domain=self.model.domain,
-                                           **self.LEGEND_OPTIONS)
+            self.legend = OWDiscreteLegend(
+                domain=self.model.domain, **self.LEGEND_OPTIONS
+            )
         else:
             items = (
-                (self.target_class_combo.itemText(self.target_class_index),
-                 self.color_palette[self.target_class_index - 1]),
-                ('other', QColor('#ffffff'))
+                (
+                    self.target_class_combo.itemText(self.target_class_index),
+                    self.color_palette[self.target_class_index - 1],
+                ),
+                ("other", QColor("#ffffff")),
             )
             self.legend = OWDiscreteLegend(items=items, **self.LEGEND_OPTIONS)
 
@@ -393,10 +428,10 @@ class OWPythagorasTree(OWWidget):
 
 
 class TreeGraphicsView(
-        PannableGraphicsView,
-        ZoomableGraphicsView,
-        AnchorableGraphicsView,
-        PreventDefaultWheelEvent
+    PannableGraphicsView,
+    ZoomableGraphicsView,
+    AnchorableGraphicsView,
+    PreventDefaultWheelEvent,
 ):
     """QGraphicsView that contains all functionality we will use to display
     tree."""
@@ -408,7 +443,8 @@ class TreeGraphicsScene(UpdateItemsOnSelectGraphicsScene):
 
 if __name__ == "__main__":  # pragma: no cover
     from Orange.modelling import TreeLearner
-    data = Table('iris')
+
+    data = Table("iris")
     # data = Table('housing')
     model = TreeLearner(max_depth=1000)(data)
     model.instances = data
