@@ -2,11 +2,28 @@
 from math import log, sqrt
 from typing import Any, Callable, Optional
 
-from AnyQt.QtCore import Qt, QRectF, QSize, QPointF, QSizeF, QModelIndex, \
-    QItemSelection, QItemSelectionModel, QT_VERSION
+from AnyQt.QtCore import (
+    Qt,
+    QRectF,
+    QSize,
+    QPointF,
+    QSizeF,
+    QModelIndex,
+    QItemSelection,
+    QItemSelectionModel,
+    QT_VERSION,
+)
 from AnyQt.QtGui import QPainter, QPen, QColor, QBrush, QMouseEvent
-from AnyQt.QtWidgets import QSizePolicy, QGraphicsScene, QLabel, QSlider, \
-    QListView, QStyledItemDelegate, QStyleOptionViewItem, QStyle
+from AnyQt.QtWidgets import (
+    QSizePolicy,
+    QGraphicsScene,
+    QLabel,
+    QSlider,
+    QListView,
+    QStyledItemDelegate,
+    QStyleOptionViewItem,
+    QStyle,
+)
 
 from Orange.base import RandomForestModel, TreeModel
 from Orange.data import Table
@@ -18,8 +35,7 @@ from Orange.widgets.visualize.pythagorastreeviewer import (
     PythagorasTreeViewer,
     ContinuousTreeNode,
 )
-from Orange.widgets.visualize.utils.tree.skltreeadapter import \
-    SklTreeAdapter
+from Orange.widgets.visualize.utils.tree.skltreeadapter import SklTreeAdapter
 from Orange.widgets.widget import OWWidget
 
 
@@ -43,22 +59,23 @@ class PythagoreanForestModel(PyListModel):
             return self.item_scale * QSize(100, 100)
 
         if role == Qt.DisplayRole:
-            if 'tree' not in self._other_data[idx]:
+            if "tree" not in self._other_data[idx]:
                 scene = QGraphicsScene(parent=self)
                 tree = PythagorasTreeViewer(
                     adapter=self._list[idx],
                     weight_adjustment=OWPythagoreanForest.SIZE_CALCULATION[
-                        self.size_calc_idx][1],
+                        self.size_calc_idx
+                    ][1],
                     interactive=False,
                     padding=100,
                     depth_limit=self.depth_limit,
                     target_class_index=self.target_class_idx,
                 )
                 scene.addItem(tree)
-                self._other_data[idx]['scene'] = scene
-                self._other_data[idx]['tree'] = tree
+                self._other_data[idx]["scene"] = scene
+                self._other_data[idx]["tree"] = tree
 
-            return self._other_data[idx]['scene']
+            return self._other_data[idx]["scene"]
 
         return super().data(index, role)
 
@@ -71,8 +88,8 @@ class PythagoreanForestModel(PyListModel):
         # type: (Callable[[PythagorasTreeViewer], None]) -> None
         """Apply `func` to every rendered tree viewer instance."""
         for idx, tree_data in enumerate(self._other_data):
-            if 'tree' in tree_data:
-                func(tree_data['tree'])
+            if "tree" in tree_data:
+                func(tree_data["tree"])
                 index = self.index(idx)
                 if QT_VERSION < 0x50000:
                     self.dataChanged.emit(index, index)
@@ -112,7 +129,7 @@ class PythagorasTreeDelegate(QStyledItemDelegate):
             painter.setPen(QPen(QColor(125, 162, 206, 192)))
             painter.setBrush(QBrush(QColor(217, 232, 252, 192)))
         else:
-            painter.setPen(QPen(QColor('#ebebeb')))
+            painter.setPen(QPen(QColor("#ebebeb")))
         painter.drawRoundedRect(rect, 3, 3)
         painter.restore()
 
@@ -146,6 +163,7 @@ class PythagorasTreeDelegate(QStyledItemDelegate):
 
 class ClickToClearSelectionListView(QListView):
     """Clicking outside any item clears the current selection."""
+
     def mousePressEvent(self, event):
         # type: (QMouseEvent) -> None
         super().mousePressEvent(event)
@@ -156,23 +174,25 @@ class ClickToClearSelectionListView(QListView):
 
 
 class OWPythagoreanForest(OWWidget):
-    name = '毕达哥拉斯森林(Pythagorean Forest)'
-    description = '毕达哥拉斯森林，用于将随机森林可视化。'
-    icon = 'icons/PythagoreanForest.svg'
+    name = "毕达哥拉斯森林(Pythagorean Forest)"
+    description = "毕达哥拉斯森林，用于将随机森林可视化。"
+    icon = "icons/PythagoreanForest.svg"
     settings_version = 2
-    keywords = ["fractal", 'bidagelasisenlin', 'gougusenlin']
-    category = 'visualize'
+    keywords = ["fractal", "bidagelasisenlin", "gougusenlin"]
+    category = "可视化(Visualize)"
 
     priority = 1001
 
     class Inputs:
-        random_forest = Input("随机森林(Random forest)", RandomForestModel, replaces=['Data'])
+        random_forest = Input(
+            "随机森林(Random forest)", RandomForestModel, replaces=["Data"]
+        )
 
     class Outputs:
-        tree = Output("树(Tree)", TreeModel, replaces=['Tree'])
+        tree = Output("树(Tree)", TreeModel, replaces=["Tree"])
 
     # Enable the save as feature
-    graph_name = 'scene'
+    graph_name = "scene"
 
     # Settings
     settingsHandler = settings.ClassValuesContextHandler()
@@ -185,19 +205,19 @@ class OWPythagoreanForest(OWWidget):
     selected_index = settings.ContextSetting(None)
 
     SIZE_CALCULATION = [
-        ('正常', lambda x: x),
-        ('平方根', lambda x: sqrt(x)),
-        ('对数的', lambda x: log(x + 1)),
+        ("正常", lambda x: x),
+        ("平方根", lambda x: sqrt(x)),
+        ("对数的", lambda x: log(x + 1)),
     ]
 
     @classmethod
     def migrate_settings(cls, settings, version):
         if version < 2:
-            settings.pop('selected_tree_index', None)
+            settings.pop("selected_tree_index", None)
             v1_min, v1_max = 20, 150
             v2_min, v2_max = 100, 400
             ratio = (v2_max - v2_min) / (v1_max - v1_min)
-            settings['zoom'] = int(ratio * (settings['zoom'] - v1_min) + v2_min)
+            settings["zoom"] = int(ratio * (settings["zoom"] - v1_min) + v2_min)
 
     def __init__(self):
         super().__init__()
@@ -209,27 +229,45 @@ class OWPythagoreanForest(OWWidget):
 
         # CONTROL AREA
         # Tree info area
-        box_info = gui.widgetBox(self.controlArea, '森林')
+        box_info = gui.widgetBox(self.controlArea, "森林")
         self.ui_info = gui.widgetLabel(box_info)
 
         # Display controls area
-        box_display = gui.widgetBox(self.controlArea, '显示')
+        box_display = gui.widgetBox(self.controlArea, "显示")
+        # maxValue is set to a wide three-digit number to probably ensure the
+        # proper label width. The maximum is later set to match the tree depth
         self.ui_depth_slider = gui.hSlider(
-            box_display, self, 'depth_limit', label='深度', ticks=False,
+            box_display, self, "depth_limit", label="深度", ticks=False, maxValue=900
         )  # type: QSlider
         self.ui_target_class_combo = gui.comboBox(
-            box_display, self, 'target_class_index', label='目标类别',
-            orientation=Qt.Horizontal, items=[], contentsLength=8,
-            searchable=True
+            box_display,
+            self,
+            "target_class_index",
+            label="目标类别",
+            orientation=Qt.Horizontal,
+            items=[],
+            contentsLength=8,
+            searchable=True,
         )
         self.ui_size_calc_combo = gui.comboBox(
-            box_display, self, 'size_calc_idx', label='大小',
+            box_display,
+            self,
+            "size_calc_idx",
+            label="大小",
             orientation=Qt.Horizontal,
-            items=list(zip(*self.SIZE_CALCULATION))[0], contentsLength=8,
+            items=list(zip(*self.SIZE_CALCULATION))[0],
+            contentsLength=8,
         )
         self.ui_zoom_slider = gui.hSlider(
-            box_display, self, 'zoom', label='缩放', ticks=False, minValue=100,
-            maxValue=400, createLabel=False, intOnly=False,
+            box_display,
+            self,
+            "zoom",
+            label="缩放",
+            ticks=False,
+            minValue=100,
+            maxValue=400,
+            createLabel=False,
+            intOnly=False,
         )  # type: QSlider
 
         # Stretch to fit the rest of the unsused area
@@ -240,14 +278,14 @@ class OWPythagoreanForest(OWWidget):
         # MAIN AREA
         self.forest_model = PythagoreanForestModel(parent=self)
         self.forest_model.update_item_size(self.zoom)
-        self.ui_depth_slider.valueChanged.connect(
-            self.forest_model.update_depth)
+        self.ui_depth_slider.valueChanged.connect(self.forest_model.update_depth)
         self.ui_target_class_combo.currentIndexChanged.connect(
-            self.forest_model.update_target_class)
-        self.ui_zoom_slider.valueChanged.connect(
-            self.forest_model.update_item_size)
+            self.forest_model.update_target_class
+        )
+        self.ui_zoom_slider.valueChanged.connect(self.forest_model.update_item_size)
         self.ui_size_calc_combo.currentIndexChanged.connect(
-            self.forest_model.update_size_calc)
+            self.forest_model.update_size_calc
+        )
 
         self.list_delegate = PythagorasTreeDelegate(parent=self)
         self.list_view = ClickToClearSelectionListView(parent=self)
@@ -291,7 +329,9 @@ class OWPythagoreanForest(OWWidget):
         if self.selected_index is not None:
             index = self.list_view.model().index(self.selected_index)
             selection = QItemSelection(index, index)
-            self.list_view.selectionModel().select(selection, QItemSelectionModel.ClearAndSelect)
+            self.list_view.selectionModel().select(
+                selection, QItemSelectionModel.ClearAndSelect
+            )
 
     def clear(self):
         """Clear all relevant data from the widget."""
@@ -305,7 +345,7 @@ class OWPythagoreanForest(OWWidget):
         self._clear_depth_slider()
 
     def _update_info_box(self):
-        self.ui_info.setText('树: {} 棵'.format(len(self.forest.trees)))
+        self.ui_info.setText("Trees: {}".format(len(self.forest.trees)))
 
     def _update_depth_slider(self):
         self.depth_limit = self._get_max_depth()
@@ -316,15 +356,18 @@ class OWPythagoreanForest(OWWidget):
 
     def _update_target_class_combo(self):
         self._clear_target_class_combo()
-        label = [x for x in self.ui_target_class_combo.parent().children()
-                 if isinstance(x, QLabel)][0]
+        label = [
+            x
+            for x in self.ui_target_class_combo.parent().children()
+            if isinstance(x, QLabel)
+        ][0]
 
         if self.instances.domain.has_discrete_class:
-            label_text = '目标类别'
+            label_text = "目标类别"
             values = [c.title() for c in self.instances.domain.class_vars[0].values]
-            values.insert(0, '无')
+            values.insert(0, "无")
         else:
-            label_text = 'Node color'
+            label_text = "Node color"
             values = list(ContinuousTreeNode.COLOR_METHODS.keys())
         label.setText(label_text)
         self.ui_target_class_combo.addItems(values)
@@ -332,7 +375,7 @@ class OWPythagoreanForest(OWWidget):
         self.target_class_index = 0
 
     def _clear_info_box(self):
-        self.ui_info.setText('没有森林输入')
+        self.ui_info.setText("No forest on input.")
 
     def _clear_target_class_combo(self):
         self.ui_target_class_combo.clear()
@@ -381,6 +424,7 @@ class OWPythagoreanForest(OWWidget):
 class SklRandomForestAdapter:
     """Take a `RandomForest` and wrap all the trees into the `SklTreeAdapter`
     instances that Pythagorean trees use."""
+
     def __init__(self, model):
         self._adapters = None
         self._domain = model.domain
@@ -399,9 +443,10 @@ class SklRandomForestAdapter:
         return self._domain
 
 
-if __name__ == '__main__':  # pragma: no cover
+if __name__ == "__main__":  # pragma: no cover
     from Orange.modelling import RandomForestLearner
-    data = Table('iris')
+
+    data = Table("iris")
     rf = RandomForestLearner(n_estimators=10)(data)
     rf.instances = data
     WidgetPreview(OWPythagoreanForest).run(rf)
