@@ -1,20 +1,24 @@
 import numpy as np
 
 from AnyQt.QtCore import (
-    Qt, QLineF, QSize, QAbstractTableModel, QModelIndex, QSortFilterProxyModel
+    Qt,
+    QLineF,
+    QSize,
+    QAbstractTableModel,
+    QModelIndex,
+    QSortFilterProxyModel,
 )
 from AnyQt.QtGui import QPainter, QPen, QBrush, QColor
-from AnyQt.QtWidgets import (
-    QItemDelegate, QHeaderView, QPushButton, QApplication
-)
+from AnyQt.QtWidgets import QItemDelegate, QHeaderView, QPushButton, QApplication
 
 from Orange.data import Table
 from Orange.classification.rules import _RuleClassifier
 from Orange.widgets import widget, gui, settings
-from Orange.widgets.utils.annotated_data import (create_annotated_table,
-                                                 ANNOTATED_DATA_SIGNAL_Chinese_NAME)
+from Orange.widgets.utils.annotated_data import (
+    create_annotated_table,
+    ANNOTATED_DATA_SIGNAL_NAME,
+)
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Input, Output
 
 
@@ -23,22 +27,24 @@ class OWRuleViewer(widget.OWWidget):
     description = "查看由数据引发的规则。"
     icon = "icons/CN2RuleViewer.svg"
     priority = 1140
-    keywords = ['cn2guize', 'guize']
-    category = 'visualize'
+    keywords = ["cn2guize", "guize"]
+    category = "可视化(Visualize)"
 
     class Inputs:
-        data = Input("数据(Data)", Table, replaces=['Data'])
-        classifier = Input("分类器(Classifier)", _RuleClassifier, replaces=['Classifier'])
+        data = Input("数据(Data)", Table, replaces=["Data"])
+        classifier = Input("分类器(Classifier)", _RuleClassifier, replaces=["Classifier"])
 
     class Outputs:
-        selected_data = Output("选定的数据(Selected Data)", Table, default=True, replaces=['Selected Data'])
-        annotated_data = Output(ANNOTATED_DATA_SIGNAL_Chinese_NAME, Table, replaces=['Data'])
+        selected_data = Output(
+            "选定的数据(Selected Data)", Table, default=True, replaces=["Selected Data"]
+        )
+        annotated_data = Output("数据(Data)", Table, replaces=["Data"])
 
     compact_view = settings.Setting(False)
 
     want_basic_layout = True
-    want_main_area = True
-    want_control_area = False
+    want_main_area = False
+    want_control_area = True
 
     def __init__(self):
         super().__init__()
@@ -47,13 +53,10 @@ class OWRuleViewer(widget.OWWidget):
         self.classifier = None
         self.selected = None
 
-        self.info.set_input_summary(self.info.NoInput)
-        self.info.set_output_summary(self.info.NoOutput)
-
         self.model = CustomRuleViewerTableModel(parent=self)
         self.model.set_horizontal_header_labels(
-            ["如果条件", "", "然后类别", "分布",
-             "概率 [%]", "质量", "长度"])
+            ["如果条件", "", "然后类别", "分布", "概率 [%]", "质量", "长度"]
+        )
 
         self.proxy_model = QSortFilterProxyModel(parent=self)
         self.proxy_model.setSourceModel(self.model)
@@ -68,25 +71,30 @@ class OWRuleViewer(widget.OWWidget):
         self.dist_item_delegate = DistributionItemDelegate(self)
         self.view.setItemDelegateForColumn(3, self.dist_item_delegate)
 
-        self.mainArea.layout().setContentsMargins(0, 0, 0, 0)
-        self.mainArea.layout().addWidget(self.view)
-        bottom_box = gui.hBox(widget=self.mainArea, box=None,
-                              margin=0, spacing=0)
+        self.controlArea.layout().addWidget(self.view)
 
-        original_order_button = QPushButton(
-            "恢复原始顺序", autoDefault=False)
-        original_order_button.setFixedWidth(180)
-        bottom_box.layout().addWidget(original_order_button)
+        gui.checkBox(
+            widget=self.buttonsArea,
+            master=self,
+            value="compact_view",
+            label="紧凑型视图",
+            callback=self.on_update,
+        )
+        gui.rubber(self.buttonsArea)
+
+        original_order_button = gui.button(
+            self.buttonsArea,
+            self,
+            "恢复原始顺序",
+            autoDefault=False,
+            callback=self.restore_original_order,
+            attribute=Qt.WA_LayoutUsesWidgetRect,
+        )
         original_order_button.clicked.connect(self.restore_original_order)
 
-        gui.separator(bottom_box, width=5, height=0)
-        gui.checkBox(widget=bottom_box, master=self, value="compact_view",
-                     label="紧凑型视图", callback=self.on_update)
-
     @Inputs.data
     def set_data(self, data):
         self.data = data
-        self._set_input_summary()
         self.commit()
 
     @Inputs.classifier
@@ -97,31 +105,30 @@ class OWRuleViewer(widget.OWWidget):
 
         if classifier is not None and hasattr(classifier, "rule_list"):
             self.model.set_vertical_header_labels(
-                list(range(len(classifier.rule_list))))
+                list(range(len(classifier.rule_list)))
+            )
 
-            self.dist_item_delegate.color_schema = \
-                [QColor(*c) for c in classifier.domain.class_var.colors]
+            self.dist_item_delegate.color_schema = [
+                QColor(*c) for c in classifier.domain.class_var.colors
+            ]
 
             self.model.wrap(self.classifier.domain, self.classifier.rule_list)
 
         self.on_update()
         self.commit()
 
-    def _set_input_summary(self):
-        summary = len(self.data) if self.data else self.info.NoInput
-        details = format_summary_details(self.data) if self.data else ""
-        self.info.set_input_summary(summary, details)
-
     def on_update(self):
         self._save_selected()
 
         self.model.set_compact_view(self.compact_view)
         if self.compact_view:
             self.view.horizontalHeader().setSectionResizeMode(
-                0, QHeaderView.Interactive)  # QHeaderView.Stretch
+                0, QHeaderView.Interactive
+            )  # QHeaderView.Stretch
         else:
             self.view.horizontalHeader().setSectionResizeMode(
-                QHeaderView.ResizeToContents)
+                QHeaderView.ResizeToContents
+            )
         self.view.resizeColumnsToContents()
         self.view.resizeRowsToContents()
 
@@ -135,18 +142,19 @@ class OWRuleViewer(widget.OWWidget):
                 selection = selection_model.selection()
             else:
                 selection = self.proxy_model.mapSelectionToSource(
-                    selection_model.selection())
+                    selection_model.selection()
+                )
 
-            self.selected = sorted(set(index.row() for index
-                                       in selection.indexes()))
+            self.selected = sorted(set(index.row() for index in selection.indexes()))
 
     def _restore_selected(self):
         if self.selected is not None:
             selection_model = self.view.selectionModel()
             for row in self.selected:
-                selection_model.select(self.proxy_model.index(row, 0),
-                                       selection_model.Select |
-                                       selection_model.Rows)
+                selection_model.select(
+                    self.proxy_model.index(row, 0),
+                    selection_model.Select | selection_model.Rows,
+                )
 
     def restore_original_order(self):
         self.proxy_model.sort(-1)
@@ -154,8 +162,9 @@ class OWRuleViewer(widget.OWWidget):
     def copy_to_clipboard(self):
         self._save_selected(actual=True)
         if self.selected is not None:
-            output = "\n".join([str(self.classifier.rule_list[i])
-                                for i in self.selected])
+            output = "\n".join(
+                [str(self.classifier.rule_list[i]) for i in self.selected]
+            )
             QApplication.clipboard().setText(output)
 
     def commit(self):
@@ -164,11 +173,12 @@ class OWRuleViewer(widget.OWWidget):
         selected_indices = []
 
         data = self.data or self.classifier and self.classifier.instances
-        if (self.selected is not None and
-                data is not None and
-                self.classifier is not None and
-                data.domain.attributes ==
-                self.classifier.original_domain.attributes):
+        if (
+            self.selected is not None
+            and data is not None
+            and self.classifier is not None
+            and data.domain.attributes == self.classifier.original_domain.attributes
+        ):
 
             status = np.ones(data.X.shape[0], dtype=bool)
             for i in self.selected:
@@ -176,12 +186,12 @@ class OWRuleViewer(widget.OWWidget):
                 status &= rule.evaluate_data(data.X)
 
             selected_indices = status.nonzero()[0]
-            data_output = data.from_table_rows(data, selected_indices) \
-                if len(selected_indices) else None
+            data_output = (
+                data.from_table_rows(data, selected_indices)
+                if len(selected_indices)
+                else None
+            )
 
-        summary = len(data_output) if data_output else self.info.NoOutput
-        details = format_summary_details(data_output) if data_output else ""
-        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(data_output)
         self.Outputs.annotated_data.send(create_annotated_table(data, selected_indices))
 
@@ -199,12 +209,7 @@ class CustomRuleViewerTableModel(QAbstractTableModel):
     SortRole = Qt.UserRole + 1
 
     # ascii operators
-    OPERATORS = {
-        '==': '=',
-        '!=': '≠',
-        '<=': '≤',
-        '>=': '≥'
-    }
+    OPERATORS = {"==": "=", "!=": "≠", "<=": "≤", ">=": "≥"}
 
     def __init__(self, parent=None):
         super().__init__(parent)
@@ -216,8 +221,9 @@ class CustomRuleViewerTableModel(QAbstractTableModel):
     def headerData(self, section, orientation, role=Qt.DisplayRole):
         if role == Qt.DisplayRole:
             headers = self._headers.get(orientation)
-            return (headers[section] if headers and section < len(headers)
-                    else str(section))
+            return (
+                headers[section] if headers and section < len(headers) else str(section)
+            )
         return None
 
     def set_horizontal_header_labels(self, labels):
@@ -256,37 +262,52 @@ class CustomRuleViewerTableModel(QAbstractTableModel):
         def _display_role():
             if column == 0:
                 delim = " AND " if self._compact_view else " AND\n"
-                return "TRUE" if not rule.selectors else delim.join(
-                    [attributes[s.column].name + self.OPERATORS[s.op] +
-                     (attributes[s.column].values[int(s.value)]
-                      if attributes[s.column].is_discrete
-                      else str(s.value)) for s in rule.selectors])
+                return (
+                    "TRUE"
+                    if not rule.selectors
+                    else delim.join(
+                        [
+                            attributes[s.column].name
+                            + self.OPERATORS[s.op]
+                            + (
+                                attributes[s.column].values[int(s.value)]
+                                if attributes[s.column].is_discrete
+                                else str(s.value)
+                            )
+                            for s in rule.selectors
+                        ]
+                    )
+                )
             if column == 1:
-                return '→'
+                return "→"
             if column == 2:
                 return class_var.name + "=" + class_var.values[rule.prediction]
             if column == 3:
                 # type(curr_class_dist) = ndarray
-                return ([float(format(x, '.1f')) for x in rule.curr_class_dist]
-                        if rule.curr_class_dist.dtype == float
-                        else rule.curr_class_dist.tolist())
+                return (
+                    [float(format(x, ".1f")) for x in rule.curr_class_dist]
+                    if rule.curr_class_dist.dtype == float
+                    else rule.curr_class_dist.tolist()
+                )
             if column == 4:
-                return " : ".join(str(int(round(100 * x)))
-                                  for x in rule.probabilities)
+                return " : ".join(str(int(round(100 * x))) for x in rule.probabilities)
             if column == 5:
                 value = rule.quality
                 absval = abs(value)
                 strlen = len(str(int(absval)))
-                return '{:.{}{}}'.format(value,
-                                         2 if absval < .001 else
-                                         3 if strlen < 2 else
-                                         1 if strlen < 5 else
-                                         0 if strlen < 6 else
-                                         3,
-                                         'f' if (absval == 0 or
-                                                 absval >= .001 and
-                                                 strlen < 6)
-                                         else 'e')
+                return "{:.{}{}}".format(
+                    value,
+                    2
+                    if absval < 0.001
+                    else 3
+                    if strlen < 2
+                    else 1
+                    if strlen < 5
+                    else 0
+                    if strlen < 6
+                    else 3,
+                    "f" if (absval == 0 or absval >= 0.001 and strlen < 6) else "e",
+                )
             if column == 6:
                 return rule.length
 
@@ -300,14 +321,28 @@ class CustomRuleViewerTableModel(QAbstractTableModel):
             if column == 3:
                 # list of int, float
                 curr_class_dist = _display_role()
-                return class_var.name + "\n" + "\n".join(
-                    (str(curr_class) + ": " + str(curr_class_dist[i])
-                     for i, curr_class in enumerate(class_var.values)))
+                return (
+                    class_var.name
+                    + "\n"
+                    + "\n".join(
+                        (
+                            str(curr_class) + ": " + str(curr_class_dist[i])
+                            for i, curr_class in enumerate(class_var.values)
+                        )
+                    )
+                )
             if column == 4:
-                return class_var.name + "\n" + "\n".join(
-                    str(curr_class) + ": " +
-                    '{:.1f}'.format(rule.probabilities[i] * 100) + "%"
-                    for i, curr_class in enumerate(class_var.values))
+                return (
+                    class_var.name
+                    + "\n"
+                    + "\n".join(
+                        str(curr_class)
+                        + ": "
+                        + "{:.1f}".format(rule.probabilities[i] * 100)
+                        + "%"
+                        for i, curr_class in enumerate(class_var.values)
+                    )
+                )
             return _display_role()
 
         def _sort_role():
@@ -330,11 +365,18 @@ class CustomRuleViewerTableModel(QAbstractTableModel):
         if role == self.SortRole:
             return _sort_role()
         if role == Qt.TextAlignmentRole:
-            return (Qt.AlignVCenter | [(Qt.AlignRight if self._compact_view
-                                        else Qt.AlignLeft), Qt.AlignCenter,
-                                       Qt.AlignLeft, Qt.AlignCenter,
-                                       Qt.AlignCenter, Qt.AlignRight,
-                                       Qt.AlignRight][column])
+            return (
+                Qt.AlignVCenter
+                | [
+                    (Qt.AlignRight if self._compact_view else Qt.AlignLeft),
+                    Qt.AlignCenter,
+                    Qt.AlignLeft,
+                    Qt.AlignCenter,
+                    Qt.AlignCenter,
+                    Qt.AlignRight,
+                    Qt.AlignRight,
+                ][column]
+            )
 
     def __len__(self):
         return len(self._rule_list)
@@ -378,7 +420,7 @@ class DistributionItemDelegate(QItemDelegate):
 
             text = str(index.data(Qt.DisplayRole))
             option.displayAlignment = Qt.AlignCenter
-            text_rect = rect.adjusted(0, 0, 0, -textoffset*0)
+            text_rect = rect.adjusted(0, 0, 0, -textoffset * 0)
             self.drawDisplay(painter, option, text_rect, text)
 
             painter.setRenderHint(QPainter.Antialiasing)
@@ -397,6 +439,7 @@ class DistributionItemDelegate(QItemDelegate):
 
 if __name__ == "__main__":  # pragma: no cover
     from Orange.classification import CN2Learner
+
     data = Table("iris")
     learner = CN2Learner()
     model = learner(data)
