@@ -2,8 +2,12 @@
 import numpy as np
 
 from AnyQt.QtWidgets import (
-    QGraphicsRectItem, QGraphicsTextItem, QSizePolicy, QStyle,
-    QLabel, QComboBox
+    QGraphicsRectItem,
+    QGraphicsTextItem,
+    QSizePolicy,
+    QStyle,
+    QLabel,
+    QComboBox,
 )
 from AnyQt.QtGui import QColor, QBrush, QPen, QFontMetrics
 from AnyQt.QtCore import Qt, QPointF, QSizeF, QRectF
@@ -12,23 +16,26 @@ from orangewidget.utils.combobox import ComboBoxSearch
 from Orange.base import TreeModel, SklModel
 from Orange.widgets.utils.signals import Input, Output
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.utils.state_summary import format_summary_details
-from Orange.widgets.visualize.owtreeviewer2d import \
-    GraphicsNode, GraphicsEdge, OWTreeViewer2D
+from Orange.widgets.visualize.owtreeviewer2d import (
+    GraphicsNode,
+    GraphicsEdge,
+    OWTreeViewer2D,
+)
 from Orange.widgets.utils import to_html
 from Orange.data import Table
 
-from Orange.widgets.settings import ContextSetting, ClassValuesContextHandler, \
-    Setting
-from Orange.widgets.utils.annotated_data import (create_annotated_table,
-                                                 ANNOTATED_DATA_SIGNAL_Chinese_NAME)
+from Orange.widgets.settings import ContextSetting, ClassValuesContextHandler, Setting
+from Orange.widgets.utils.annotated_data import (
+    create_annotated_table,
+    ANNOTATED_DATA_SIGNAL_NAME,
+)
 from Orange.widgets.visualize.utils.tree.skltreeadapter import SklTreeAdapter
 from Orange.widgets.visualize.utils.tree.treeadapter import TreeAdapter
 
 
 class PieChart(QGraphicsRectItem):
-    """PieChart graphics item added at the corner of classification tree nodes
-    """
+    """PieChart graphics item added at the corner of classification tree nodes"""
+
     # Methods are documented in PyQt documentation
     # pylint: disable=missing-docstring
     def __init__(self, dist, r, parent):
@@ -52,13 +59,14 @@ class PieChart(QGraphicsRectItem):
         start_angle = 0
         colors = self.scene().colors
         for i in range(len(self.dist)):
-            angle = self.dist[i] * 16 * 360. / dist_sum
+            angle = self.dist[i] * 16 * 360.0 / dist_sum
             if angle == 0:
                 continue
             painter.setBrush(QBrush(colors[i]))
             painter.setPen(QPen(colors[i]))
-            painter.drawPie(-self.r, -self.r, 2 * self.r, 2 * self.r,
-                            int(start_angle), int(angle))
+            painter.drawPie(
+                -self.r, -self.r, 2 * self.r, 2 * self.r, int(start_angle), int(angle)
+            )
             start_angle += angle
         painter.setPen(QPen(Qt.black))
         painter.setBrush(QBrush())
@@ -67,6 +75,7 @@ class PieChart(QGraphicsRectItem):
 
 class TreeNode(GraphicsNode):
     """TreeNode for trees corresponding to base.Tree models"""
+
     # Methods are documented in PyQt documentation
     # pylint: disable=missing-docstring
 
@@ -78,13 +87,15 @@ class TreeNode(GraphicsNode):
 
         fm = QFontMetrics(self.document().defaultFont())
         attr = self.tree_adapter.attribute(node_inst)
-        self.attr_text_w = fm.width(attr.name if attr else "")
+        self.attr_text_w = fm.horizontalAdvance(attr.name if attr else "")
         self.attr_text_h = fm.lineSpacing()
         self.line_descent = fm.descent()
         self._rect = None
 
         if self.model.domain.class_var.is_discrete:
-            self.pie = PieChart(self.tree_adapter.get_distribution(node_inst)[0], 8, self)
+            self.pie = PieChart(
+                self.tree_adapter.get_distribution(node_inst)[0], 8, self
+            )
         else:
             self.pie = None
 
@@ -96,7 +107,7 @@ class TreeNode(GraphicsNode):
         self.droplet.setVisible(bool(self.branches))
         fm = QFontMetrics(self.document().defaultFont())
         attr = self.tree_adapter.attribute(self.node_inst)
-        self.attr_text_w = fm.width(attr.name if attr else "")
+        self.attr_text_w = fm.horizontalAdvance(attr.name if attr else "")
         self.attr_text_h = fm.lineSpacing()
         self.line_descent = fm.descent()
         if self.pie is not None:
@@ -106,9 +117,9 @@ class TreeNode(GraphicsNode):
         if self._rect and self._rect.isValid():
             return self._rect
         else:
-            return QRectF(QPointF(0, 0), self.document().size()).\
-                adjusted(0, 0, 8, 0) | \
-                (getattr(self, "_rect") or QRectF(0, 0, 1, 1))
+            return QRectF(QPointF(0, 0), self.document().size()).adjusted(
+                0, 0, 8, 0
+            ) | (getattr(self, "_rect") or QRectF(0, 0, 1, 1))
 
     def set_rect(self, rect):
         self.prepareGeometryChange()
@@ -120,11 +131,13 @@ class TreeNode(GraphicsNode):
 
     def boundingRect(self):
         if hasattr(self, "attr"):
-            attr_rect = QRectF(QPointF(0, -self.attr_text_h),
-                               QSizeF(self.attr_text_w, self.attr_text_h))
+            attr_rect = QRectF(
+                QPointF(0, -self.attr_text_h),
+                QSizeF(self.attr_text_w, self.attr_text_h),
+            )
         else:
             attr_rect = QRectF(0, 0, 1, 1)
-        rect = self.rect().adjusted(-5, -5, 5, 5)
+        rect = self.rect().adjusted(-6, -6, 6, 6)
         return rect | attr_rect
 
     def paint(self, painter, option, widget=None):
@@ -137,13 +150,17 @@ class TreeNode(GraphicsNode):
             draw_text = str(self.tree_adapter.short_rule(self.node_inst))
             if self.parent.x() > self.x():  # node is to the left
                 fm = QFontMetrics(font)
-                x = rect.width() / 2 - fm.width(draw_text) - 4
+                x = rect.width() / 2 - fm.horizontalAdvance(draw_text) - 4
             else:
                 x = rect.width() / 2 + 4
             painter.drawText(QPointF(x, -self.line_descent - 1), draw_text)
         painter.save()
         painter.setBrush(self.backgroundBrush)
-        painter.setPen(QPen(Qt.black, 3 if self.isSelected() else 0))
+        if self.isSelected():
+            outline = QPen(option.palette.highlight(), 3)
+        else:
+            outline = QPen(option.palette.dark(), 1)
+        painter.setPen(outline)
         adjrect = rect.adjusted(-3, 0, 0, 0)
         if not self.tree_adapter.has_children(self.node_inst):
             painter.drawRoundedRect(adjrect, 4, 4)
@@ -161,17 +178,32 @@ class OWTreeGraph(OWTreeViewer2D):
     icon = "icons/TreeViewer.svg"
     description = "分类树和回归树的可视化。"
     priority = 35
-    keywords = ['chakanshu']
-    category = 'visualize'
+    keywords = ["chakanshu"]
+    category = "可视化(Visualize)"
 
     class Inputs:
         # Had different input names before merging from
         # Classification/Regression tree variants
-        tree = Input("树(Tree)", TreeModel, replaces=["Classification Tree", "Regression Tree", 'Tree'])
+        tree = Input(
+            "树(Tree)",
+            TreeModel,
+            replaces=["Classification Tree", "Regression Tree", "Tree"],
+        )
 
     class Outputs:
-        selected_data = Output("选定的数据(Selected Data)", Table, default=True, id="selected-data", replaces=['Selected Data'])
-        annotated_data = Output(ANNOTATED_DATA_SIGNAL_Chinese_NAME, Table, id="annotated-data", replaces=['Data'])
+        selected_data = Output(
+            "选定的数据(Selected Data)",
+            Table,
+            default=True,
+            id="selected-data",
+            replaces=["Selected Data"],
+        )
+        annotated_data = Output(
+            "数据(Data)",
+            Table,
+            id="annotated-data",
+            replaces=["Data"],
+        )
 
     settingsHandler = ClassValuesContextHandler()
     target_class_index = ContextSetting(0)
@@ -179,7 +211,7 @@ class OWTreeGraph(OWTreeViewer2D):
 
     replaces = [
         "Orange.widgets.classify.owclassificationtreegraph.OWClassificationTreeGraph",
-        "Orange.widgets.classify.owregressiontreegraph.OWRegressionTreeGraph"
+        "Orange.widgets.classify.owregressiontreegraph.OWRegressionTreeGraph",
     ]
 
     COL_OPTIONS = ["默认", "实例数", "平均值", "方差"]
@@ -195,14 +227,11 @@ class OWTreeGraph(OWTreeViewer2D):
         self.color_label = QLabel("目标类别:")
         combo = self.color_combo = ComboBoxSearch()
         combo.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed)
-        combo.setSizeAdjustPolicy(
-            QComboBox.AdjustToMinimumContentsLengthWithIcon)
+        combo.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLengthWithIcon)
         combo.setMinimumContentsLength(8)
         combo.activated[int].connect(self.color_changed)
         self.display_box.layout().addRow(self.color_label, combo)
 
-        self.info.set_output_summary(self.info.NoOutput)
-
     def set_node_info(self):
         """Set the content of the node"""
         for node in self.scene.nodes():
@@ -267,7 +296,7 @@ class OWTreeGraph(OWTreeViewer2D):
         self.model = model
         self.target_class_index = 0
         if model is None:
-            self.infolabel.setText('No tree.')
+            self.infolabel.setText("没有树.")
             self.root_node = None
             self.dataset = None
             self.tree_adapter = None
@@ -287,17 +316,19 @@ class OWTreeGraph(OWTreeViewer2D):
                 self.color_combo.addItems(self.domain.class_vars[0].values)
                 self.color_combo.setCurrentIndex(self.target_class_index)
             else:
-                self.color_label.setText("颜色依据: ")
+                self.color_label.setText("Color by: ")
                 self.color_combo.addItems(self.COL_OPTIONS)
                 self.color_combo.setCurrentIndex(self.regression_colors)
             self.openContext(self.domain.class_var)
             # self.root_node = self.walkcreate(model.root, None)
             self.root_node = self.walkcreate(self.tree_adapter.root)
-            self.infolabel.setText('{} 个节点, {} 片叶子'.format(
-                self.tree_adapter.num_nodes,
-                len(self.tree_adapter.leaves(self.tree_adapter.root))))
+            self.infolabel.setText(
+                "{} 个节点, {} 片叶子".format(
+                    self.tree_adapter.num_nodes,
+                    len(self.tree_adapter.leaves(self.tree_adapter.root)),
+                )
+            )
         self.setup_scene()
-        self.info.set_output_summary(self.info.NoOutput)
         self.Outputs.selected_data.send(None)
         self.Outputs.annotated_data.send(create_annotated_table(self.dataset, []))
 
@@ -315,31 +346,42 @@ class OWTreeGraph(OWTreeViewer2D):
         return node_obj
 
     def node_tooltip(self, node):
-        return "<br>".join(to_html(str(rule))
-                           for rule in self.tree_adapter.rules(node.node_inst))
+        return (
+            "<p>"
+            + "<br>".join(
+                to_html(str(rule)) for rule in self.tree_adapter.rules(node.node_inst)
+            )
+            + "</p>"
+        )
 
     def update_selection(self):
         if self.model is None:
             return
-        nodes = [item.node_inst for item in self.scene.selectedItems()
-                 if isinstance(item, TreeNode)]
+        nodes = [
+            item.node_inst
+            for item in self.scene.selectedItems()
+            if isinstance(item, TreeNode)
+        ]
         data = self.tree_adapter.get_instances_in_nodes(nodes)
 
-        summary = len(data) if data else self.info.NoOutput
-        details = format_summary_details(data) if data else ""
-        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(data)
-        self.Outputs.annotated_data.send(create_annotated_table(
-            self.dataset, self.tree_adapter.get_indices(nodes)))
+        self.Outputs.annotated_data.send(
+            create_annotated_table(self.dataset, self.tree_adapter.get_indices(nodes))
+        )
 
     def send_report(self):
         if not self.model:
             return
-        items = [("Tree size", self.infolabel.text()),
-                 ("Edge widths",
-                  ("Fixed", "Relative to root", "Relative to parent")[
-                      # pylint: disable=invalid-sequence-index
-                      self.line_width_method])]
+        items = [
+            ("Tree size", self.infolabel.text()),
+            (
+                "Edge widths",
+                ("Fixed", "Relative to root", "Relative to parent")[
+                    # pylint: disable=invalid-sequence-index
+                    self.line_width_method
+                ],
+            ),
+        ]
         if self.domain.class_var.is_discrete:
             items.append(("Target class", self.color_combo.currentText()))
         elif self.regression_colors != self.COL_DEFAULT:
@@ -372,8 +414,7 @@ class OWTreeGraph(OWTreeViewer2D):
             text += f"{100 * tabs:2.1f}%, {int(total * tabs)}/{total}"
 
         text = self._update_node_info_attr_name(node, text)
-        node.setHtml(
-            f'<p style="line-height: 120%; margin-bottom: 0">{text}</p>')
+        node.setHtml(f'<p style="line-height: 120%; margin-bottom: 0">{text}</p>')
 
     def update_node_info_reg(self, node):
         """Update the printed contents of the node for regression trees"""
@@ -383,8 +424,7 @@ class OWTreeGraph(OWTreeViewer2D):
         text = f"<b>{mean:.1f}</b> ± {var:.1f}<br/>"
         text += f"{insts} instances"
         text = self._update_node_info_attr_name(node, text)
-        node.setHtml(
-            f'<p style="line-height: 120%; margin-bottom: 0">{text}</p>')
+        node.setHtml(f'<p style="line-height: 120%; margin-bottom: 0">{text}</p>')
 
     def toggle_node_color_cls(self):
         """Update the node color for classification trees"""
@@ -394,13 +434,12 @@ class OWTreeGraph(OWTreeViewer2D):
             total = sum(distr)
             if self.target_class_index:
                 p = distr[self.target_class_index - 1] / total
-                color = colors[self.target_class_index - 1].lighter(
-                    200 - 100 * p)
+                color = colors[self.target_class_index - 1].lighter(int(200 - 100 * p))
             else:
                 modus = np.argmax(distr)
                 p = distr[modus] / (total or 1)
                 color = colors.value_to_qcolor(int(modus))
-                color = color.lighter(300 - 200 * p)
+                color = color.lighter(int(300 - 200 * p))
             node.backgroundBrush = QBrush(color)
         self.scene.update()
 
@@ -412,13 +451,16 @@ class OWTreeGraph(OWTreeViewer2D):
             for node in self.scene.nodes():
                 node.backgroundBrush = brush
         elif self.regression_colors == self.COL_INSTANCE:
-            max_insts = len(self.tree_adapter.get_instances_in_nodes(
-                [self.tree_adapter.root]))
+            max_insts = len(
+                self.tree_adapter.get_instances_in_nodes([self.tree_adapter.root])
+            )
             for node in self.scene.nodes():
-                node_insts = len(self.tree_adapter.get_instances_in_nodes(
-                    [node.node_inst]))
-                node.backgroundBrush = QBrush(def_color.lighter(
-                    120 - 20 * node_insts / max_insts))
+                node_insts = len(
+                    self.tree_adapter.get_instances_in_nodes([node.node_inst])
+                )
+                node.backgroundBrush = QBrush(
+                    def_color.lighter(int(120 - 20 * node_insts / max_insts))
+                )
         elif self.regression_colors == self.COL_MEAN:
             minv = np.nanmin(self.dataset.Y)
             maxv = np.nanmax(self.dataset.Y)
@@ -429,12 +471,15 @@ class OWTreeGraph(OWTreeViewer2D):
                 node.backgroundBrush = QBrush(color)
         else:
             nodes = list(self.scene.nodes())
-            variances = [self.tree_adapter.get_distribution(node.node_inst)[0][1]
-                         for node in nodes]
+            variances = [
+                self.tree_adapter.get_distribution(node.node_inst)[0][1]
+                for node in nodes
+            ]
             max_var = max(variances)
             for node, var in zip(nodes, variances):
-                node.backgroundBrush = QBrush(def_color.lighter(
-                    120 - 20 * var / max_var))
+                node.backgroundBrush = QBrush(
+                    def_color.lighter(int(120 - 20 * var / max_var))
+                )
         self.scene.update()
 
     def _get_tree_adapter(self, model):
@@ -445,6 +490,7 @@ class OWTreeGraph(OWTreeViewer2D):
 
 if __name__ == "__main__":  # pragma: no cover
     from Orange.modelling.tree import TreeLearner
+
     data = Table("titanic")
     # data = Table("housing")
     clf = TreeLearner()(data)
